clear
capture log close
set more on

#delimit ;

log using wt,replace;

use SEGmodel2EST.dta;

drop ma1-ma3;

** JATOmodel variable without gaps;

egen model_group1=group(model_group);
drop model_group;
rename model_group1 model_group;

*** Generate index vectors ;

*** Drop models for which there is only 1 id;

egen x = count(1), by(model_group);
drop if x==1;

*** Generate mean and deviation from the mean;

*** Deviation from the mean of all exogenous variables, princ, conditional market shares and the dependent variable;

local var "hp eurokm wi he nhome taxprinc lsj lsjg
		  lsjg_Fclass1 lsjg_Fclass2 lsjg_Fclass3 lsjg_Fclass4 lsjg_Fclass5";

foreach x in `var' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};


local country "ma";

foreach x in `country'{;
tab ma, gen(ma`y');
forvalues y = 1/3{;
egen m`x'`y'=mean(`x'`y'), by (model_group);
gen d`x'`y'=`x'`y'-m`x'`y';
drop m`x'`y';
};
};


* Deviation from the mean of multiplicative dummy variables *;

*** Trend/time controls;

tab yema, gen(yema);
local var " time1	time2	time3	 	
			time21	time22	time23
			yema1 	yema2	yema3	yema4	yema5	yema6	yema7	yema8	yema9	yema10
			yema11	yema12	yema13	yema14	yema15	yema16	yema17	yema18	yema19	yema20
			yema21	yema22	yema23	yema24	yema25	yema26	yema27	yema28	yema29	yema30
			yema31	yema32	yema33	yema34	yema35	yema36	yema37	yema38	yema39	yema40
			yema41	yema42 ";

foreach x in `var' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};

*** Month of introduction dummy;

drop md*; rename  market_duration md; label variable md "number of months the car is in the market";

local md "md";

foreach x in `md'{; 	
tab `md', gen(md`y');
forvalues y = 1/12{;
egen m`x'`y'=mean(`x'`y'), by (model_group);
gen d`x'`y'=md`y'-m`x'`y';
drop m`x'`y';
};
};


* Instruments;


*** Instruments;

local 1nl_inst  	" 					steelcontent lulc localproduct
									
										distancesq_hp1 distancesq_hp2 distancesq_hp3 distancesq_hp4 distancesq_hp5
										distancesq_li1 distancesq_li2 distancesq_li3 distancesq_li4 distancesq_li5
										distancesq_wi1 distancesq_wi2 distancesq_wi3 distancesq_wi4 distancesq_wi5
										distancesq_he1 distancesq_he2 distancesq_he3 distancesq_he4 distancesq_he5
										
										ONdistancesq_hp0 ONdistancesq_hp1 ONdistancesq_hp2 ONdistancesq_hp3 ONdistancesq_hp4 ONdistancesq_hp5 ONdistancesq_hp6
										ONdistancesq_li0 ONdistancesq_li1 ONdistancesq_li2 ONdistancesq_li3 ONdistancesq_li4 ONdistancesq_li5 ONdistancesq_li6
										ONdistancesq_wi0 ONdistancesq_wi1 ONdistancesq_wi2 ONdistancesq_wi3 ONdistancesq_wi4 ONdistancesq_wi5 ONdistancesq_wi6
										ONdistancesq_he0 ONdistancesq_he1 ONdistancesq_he2 ONdistancesq_he3 ONdistancesq_he4 ONdistancesq_he5 ONdistancesq_he6
										
										CFis_hp 	CFis_li 	CFis_wi 	CFis_he 	CFis_num
										ifs_hp 		ifs_li 		ifs_wi 		ifs_he 		ifs_num
										FONis_hp 	FONis_li 	FONis_wi 	FONis_he  	FONis_num
										FONifs_hp 	FONifs_li 	FONifs_wi 	FONifs_he  	FONifs_num";										

foreach x in `1nl_inst' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};


			
* Index vector for market ;

egen MKTSeg=group(yema Fclass);

*** Keep the variables I need for MATLAB;

keep 	ma model_group model model_jato version_jato md body year firm  brand q  gp taxprinc  population gdppercapita price_retail_adjusted_eurobp Fclass  household
		origin sj s0 co2 fuel
		sg
		sjg
		lsj lsjg
		dlsj
		lsjg dlsjg
		Fclass1 Fclass2 Fclass3 Fclass4 Fclass5
		lsjg_Fclass1 lsjg_Fclass2 lsjg_Fclass3 lsjg_Fclass4 lsjg_Fclass5
		dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5
		yema MKTSeg
		hp  eurokm  wi he size li nhome taxprinc  md1-md12  ma1 ma2 ma3  yema1-yema42
		dhp  deurokm  dwi dhe  dnhome  dtaxprinc  dmd*  dma*  dyema1-dyema42
		dtime1	dtime2	dtime3	 	
		dtime21	dtime22	dtime23

		dsteelcontent dlulc dlocalproduct
									
		ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
		ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
		ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
		ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
										
	    dONdistancesq_hp0 dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5 dONdistancesq_hp6
		dONdistancesq_li0 dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 dONdistancesq_li6
		dONdistancesq_wi0 dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5 dONdistancesq_wi6
		dONdistancesq_he0 dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5 dONdistancesq_he6	
		
		dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
		difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
		dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
		dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num;																			
						
*** Regression with demeaned variables (std. errors need to be corrected - see Matlab);

#delimit ;

ivregress gmm dlsj	(dtaxprinc 			dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5 =
										dsteelcontent dlulc dlocalproduct
									
										ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
										ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
										ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
										ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
																		
										dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5
										dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 
										dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5
										dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5
										
										dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
										difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
										dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
										dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num)
						dhp deurokm dwi dhe dnhome
						dmd1 dmd2 dmd3 dmd4 dmd5 dmd6 dmd7 dmd8 dmd9 dmd10 dmd11 	
						dyema1-dyema41 ;
											
#delimit cr
* Form a balanced dataset
egen model2=group(model), label
sort yema Fclass model2
drop model
rename model2 model

xtset model yema
tsset model yema
tsfill, full

sort model Fclass
by model: replace Fclass=Fclass[_n-1] if missing(Fclass)


gen misid=1 if taxprinc!=.
replace misid=0 if misid==.
order misid
rename size carsize

* I created a "balanced" dataset in Stata by filling the missings the unbalancedness is ignorable (the missingness is unrelated to the
* dependent variable) so the missing data can be zeroed-out all methods for
* balanced data go through - zeroed out means replacing NaN with zeros

mvencode _all, mv(0) override


order yema Fclass model

#delimit ;
ivregress 2sls dlsj	(dtaxprinc 			dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5 =
										dsteelcontent dlulc dlocalproduct
									
										ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
										ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
										ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
										ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
																		
										dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5
										dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 
										dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5
										dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5
										
										dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
										difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
										dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
										dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num)
						dhp deurokm dwi dhe dnhome
						dmd1 dmd2 dmd3 dmd4 dmd5 dmd6 dmd7 dmd8 dmd9 dmd10 dmd11
						dyema1-dyema41 ;
											
#delimit cr

* Very important!
sort yema Fclass model

log close

compress

saveold WTSeg, replace

use WTSeg.dta
outsheet using WTSeg, nolabel replace
