clear
capture log close
set more on


log using wt,replace

use model2INST.dta


***** 
* Save 100 datasets for bootstrap of the difference in the objective functions (in Matlab)
*****

local j = 1
while `j' <= 100 {
	quietly {
		
	preserve
	noisily  di in yellow  "." _continue
	

	/* bsample= Sampling with replacement in a panel */
	bsample, cluster(yema) idcluster(newid)
	rename yema oldyema
	rename newid yema
	
	sort yema Fclass model
	
***********************************************************************************************************************
*Part 1. INSTRUMENTS
***********************************************************************************************************************
local instruments "hp li wi he le eurokm size nhome"

gen potential_mkt = household - ShareExcluded

local potential_mkt "potential_mkt"

sum `characteristics'

***********************************************************************************************************************
* LOGIT INSTRUMENTS

* Sum of the characteristics of all competing products and competing products produced by other firms

* Per year and market;

/* 	i_hp= sum(hp)-hp 					sum of the characteristics of all competing products
	if_hp= sum(hp of firm j) - hp  		sum of the characteristics of all other makes of firm j
	CFi_hp= sum(hp)- sum(hp of firm j)	sum of the characteristics of all other non-firm j makes */

foreach inst in `instruments'{
qui{
	egen i_`inst'=sum(`inst'), by (year ma)
	egen if_`inst'=sum(`inst'), by (year ma firm)
	gen CFi_`inst'=i_`inst'-if_`inst'
	replace i_`inst'=i_`inst'-`inst'
	replace if_`inst'=if_`inst'-`inst'
}
}
qui egen tot_sales 	= sum(q), by(year ma)
qui egen i_num 		= count(q), by(year ma)

qui egen firm_sales = sum(q), by(year ma firm)
qui egen if_num 	= count(q), by(year ma firm)

qui gen CFi_num=i_num-if_num

replace i_num=i_num-1
gen lni_num=ln(i_num)

replace if_num=if_num-1
gen lnif_num=ln(if_num)

***********************************************************************************************************************
* ONE LEVEL NESTED LOGIT INSTRUMENTS, WHERE SEGMENT IS THE PRINCIPLE OF DIFFERENTIATION

* Per year market firm and nest

*** 	is_hp= sum(hp in nest k)-hp 								  	sum of the characteristics of all competing products in nest k
***	    ifs_hp= sum(hp of firm j in nest k) - hp  					sum of the characteristics of all other makes of firm j in nest k
***	    CFis_hp= sum(hp in nest k)- sum(hp of firm j in nest k)	sum of the characteristics of all other non-firm j makes in nest k


foreach inst in `instruments'{
qui{
	egen is_`inst' 		= sum(`inst'),  by (year ma Fclass)
	egen ifs_`inst'	 	= sum(`inst'), by (year ma Fclass firm)
	gen CFis_`inst' 	= is_`inst'-ifs_`inst'
	replace is_`inst' 	= is_`inst'-`inst'
	replace ifs_`inst' 	= ifs_`inst'-`inst'
}
}

egen seg_sales 	= sum(q), by(year ma Fclass)
egen is_num 	= count(q), by(year  ma Fclass)

egen fs_sales 	= sum(q), by(year ma Fclass firm)
egen ifs_num 	= count (q), by(year ma Fclass firm)

qui gen CFis_num = is_num-ifs_num
replace is_num 	= is_num-1
gen lnis_num 	= ln(is_num)
replace ifs_num = ifs_num-1
gen lnifs_num 	= ln(ifs_num)

***********************************************************************************************************************

* CREATION OF THE DEPENDENT VARIABLES

gen lsj=ln(q)-ln(`potential_mkt'-tot_sales) 	
/*dependent, pay attention to the name that is a bit inappropiate: ln(sk)-ln(s0)*/

gen lsjg 	 = ln(q)-ln(seg_sales)

gen lsg 	 = ln(seg_sales)-ln(`potential_mkt')

gen sj  = q/(`potential_mkt')
gen s0 	= (`potential_mkt'-tot_sales)/`potential_mkt'
gen sjg = exp(lsjg)
gen sg  = exp(lsg)

gen sj2=sjg*sg
sum sj sj2
drop sj2

gen lsj2=ln(sj)-ln(s0) 
sum lsj lsj2
drop lsj2

********** FLEXIBLE SPECIFICATION FOR FCLASS

*** Interaction only with F class

tab Fclass, gen(Fclass)

local help "is_hp is_li is_wi is_he is_size is_nhome is_num CFis_hp CFis_li CFis_wi CFis_he CFis_size CFis_nhome CFis_num  ifs_hp ifs_li ifs_wi ifs_he ifs_size ifs_nhome ifs_num"

foreach x in `help'{
	forvalues y = 1/5{
	gen `x'_Fclass`y'=`x'* Fclass`y'
}
}

*** Generate iteraction between conditional market shares and segment

local help "lsjg"

foreach x in `help'{
forvalues y = 1/5{
qui gen `x'_Fclass`y'=`x'* Fclass`y'
}
}

***********************************************************************************************************************

* ONE LEVEL NESTED LOGIT INSTRUMENTS FOR ORDERED NESTS

* M=2

gen FONclass0 = 1 if Fclass==1
gen FONclass1 = 1 if Fclass==1 | Fclass==2
gen FONclass2 = 1 if Fclass==1 | Fclass==2 | Fclass==3
gen FONclass3 = 1 if Fclass==2 | Fclass==3 | Fclass==4
gen FONclass4 = 1 if Fclass==3 | Fclass==4 | Fclass==5
gen FONclass5 = 1 if Fclass==4 | Fclass==5
gen FONclass6 = 1 if Fclass==5


qui{
foreach inst in `instruments'{
forv num =0/6{
	egen 	FONis_`inst'`num'	=  sum(`inst'),  by (year ma FONclass`num')
	replace FONis_`inst'`num'	= (FONis_`inst'`num' * FONclass`num')-`inst'
	egen 	FONifs_`inst'`num'	= sum(`inst'), by (year ma FONclass`num' firm)
	replace FONifs_`inst'`num'	= (FONifs_`inst'`num' * FONclass`num')-`inst'	
}
}
foreach inst in `instruments'{
	egen FONis_`inst' 		= rowtotal(FONis_`inst'0 FONis_`inst'1 FONis_`inst'2 FONis_`inst'3 FONis_`inst'4 FONis_`inst'5 FONis_`inst'6)
	egen FONifs_`inst' 		= rowtotal(FONifs_`inst'0 FONifs_`inst'1 FONifs_`inst'2 FONifs_`inst'3 FONifs_`inst'4 FONifs_`inst'5 FONifs_`inst'6)

}

forv num =0/6{
egen FONis_num`num'	   = count(q), by(year ma FONclass`num')
replace FONis_num`num' = FONis_num`num'	  -1
replace FONis_num`num' = FONis_num`num' * FONclass`num'

}
egen FONis_num 	= rowtotal(FONis_num0 FONis_num1 FONis_num2 FONis_num3 FONis_num4 FONis_num5 FONis_num6) 

forv num =0/6{
egen FONifs_num`num'	= count(q), by(year ma firm FONclass`num')
replace FONifs_num`num'	= FONifs_num`num'-1
replace FONifs_num`num' = FONifs_num`num' * FONclass`num'
}
}
egen FONifs_num 	= rowtotal(FONifs_num0 FONifs_num1 FONifs_num2 FONifs_num3 FONifs_num4 FONifs_num5 FONifs_num6) 

#delimit ;
mvencode    FONis_hp0 FONis_hp1 FONis_hp2 FONis_hp3 FONis_hp4 FONis_hp5 FONis_hp6   
			FONis_li0 FONis_li1 FONis_li2 FONis_li3 FONis_li4 FONis_li5 FONis_li6  
			FONis_wi0 FONis_wi1 FONis_wi2 FONis_wi3 FONis_wi4 FONis_wi5 FONis_wi6
			FONis_he0 FONis_he1 FONis_he2 FONis_he3 FONis_he4 FONis_he5 FONis_he6
			FONis_nhome0 FONis_nhome1 FONis_nhome2 FONis_nhome3 FONis_nhome4 FONis_nhome5 FONis_nhome6
			FONis_size0 FONis_size1 FONis_size2 FONis_size3 FONis_size4 FONis_size5 FONis_size6 
			FONis_num0 FONis_num1 FONis_num2 FONis_num3 FONis_num4 FONis_num5 FONis_num6

			FONifs_hp0 FONifs_hp1 FONifs_hp2 FONifs_hp3 FONifs_hp4 FONifs_hp5 FONifs_hp6
			FONifs_li0 FONifs_li1 FONifs_li2 FONifs_li3 FONifs_li4 FONifs_li5 FONifs_li6  
			
			FONifs_wi0 FONifs_wi1 FONifs_wi2 FONifs_wi3 FONifs_wi4 FONifs_wi5 FONifs_wi6 
			FONifs_he0 FONifs_he1 FONifs_he2 FONifs_he3 FONifs_he4 FONifs_he5 FONifs_he6
			FONifs_size0 FONifs_size1 FONifs_size2 FONifs_size3 FONifs_size4 FONifs_size5 FONifs_size6   
			FONifs_nhome0 FONifs_nhome1 FONifs_nhome2 FONifs_nhome3 FONifs_nhome4 FONifs_nhome5 FONifs_nhome6
			FONifs_num0 FONifs_num1 FONifs_num2 FONifs_num3 FONifs_num4 FONifs_num5 FONifs_num6
			FONis_hp FONis_li FONis_wi FONis_nhome FONis_num
			FONifs_hp FONifs_li FONifs_wi FONifs_nhome FONifs_num, mv(0) override;
			
#delimit cr		


******************************************************************

*** Gandhi and Houde instrument based on distance

*** sum (x1 - average in a group) = 1/(N-1) sum(xi-xj)

foreach inst in `instruments'{
	forvalues y = 1/5{
		egen 	 sumdistance_`inst'`y' 	= total(`inst' * Fclass`y'), by(year ma Fclass`y')
		mvdecode sumdistance_`inst'`y', mv(0)
		gen  distance_`inst'`y' 	= (`inst' - sumdistance_`inst'`y')/is_num_Fclass`y'
		
		egen sumcharsquares_`inst'`y'  = total((`inst')^2 * Fclass`y'), by(year ma Fclass`y')
		gen distancesq_`inst'`y' 		= (((is_num_Fclass`y'+1)/is_num_Fclass`y') *(`inst')^2) - ((2 * `inst' * sumdistance_`inst'`y')/(is_num_Fclass`y')) + (sumcharsquares_`inst'`y'/(is_num_Fclass`y'))
		mvencode distance_`inst'`y' distancesq_`inst'`y', mv(0) override
		drop sumdistance_`inst'`y' sumcharsquares_`inst'`y'
	}
}


foreach inst in `instruments'{
	forvalues y = 0/6{
		egen 	 ONsumdistance_`inst'`y' 	= total(`inst'* FONclass`y'), by(year ma FONclass`y')
		gen  	 ONdistance_`inst'`y' 		= (`inst' - ONsumdistance_`inst'`y')/FONis_num`y'
		mvencode ONdistance_`inst'`y', mv(0) override
		
		egen ONsumcharsquares_`inst'`y'  = total((`inst')^2), by(year ma FONclass`y')
		
		gen ONdistancesq_`inst'`y' 		= (((FONis_num`y'+1)/FONis_num`y') *(`inst')^2) - ((2 * `inst' * ONsumdistance_`inst'`y')/(FONis_num`y')) + (ONsumcharsquares_`inst'`y'/(FONis_num`y'))
		mvencode ONdistance_`inst'`y' ONdistancesq_`inst'`y', mv(0) override
		drop 	 ONsumdistance_`inst'`y' ONsumcharsquares_`inst'`y'

	}
}


compress

#delimit ;

drop ma1-ma3;

** JATOmodel variable without gaps;

egen model_group1=group(model_group);
drop model_group;
rename model_group1 model_group;

*** Generate index vectors ;

*** Drop models for which there is only 1 id;

egen x = count(1), by(model_group);
drop if x==1;

*** Generate mean and deviation from the mean;

*** Deviation from the mean of all exogenous variables, princ, conditional market shares and the dependent variable;

local var "hp eurokm wi he nhome taxprinc lsj lsjg
		  lsjg_Fclass1 lsjg_Fclass2 lsjg_Fclass3 lsjg_Fclass4 lsjg_Fclass5";

foreach x in `var' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};


local country "ma";

foreach x in `country'{;
tab ma, gen(ma`y');
forvalues y = 1/3{;
egen m`x'`y'=mean(`x'`y'), by (model_group);
gen d`x'`y'=`x'`y'-m`x'`y';
drop m`x'`y';
};
};


* Deviation from the mean of multiplicative dummy variables *;

*** Trend/time controls;

tab yema, gen(yema);
local var " time1	time2	time3	 	
			time21	time22	time23
			yema1 	yema2	yema3	yema4	yema5	yema6	yema7	yema8	yema9	yema10
			yema11	yema12	yema13	yema14	yema15	yema16	yema17	yema18	yema19	yema20
			yema21	yema22	yema23	yema24	yema25	yema26	yema27	yema28	yema29	yema30
			yema31	yema32	yema33	yema34	yema35	yema36	yema37	yema38	yema39	yema40
			yema41	yema42 ";

foreach x in `var' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};

*** Month of introduction dummy;

drop md*; rename  market_duration md; label variable md "number of months the car is in the market";

local md "md";

foreach x in `md'{; 	
tab `md', gen(md`y');
forvalues y = 1/12{;
egen m`x'`y'=mean(`x'`y'), by (model_group);
gen d`x'`y'=md`y'-m`x'`y';
drop m`x'`y';
};
};


* Instruments;


*** Instruments;

local 1nl_inst  	" 					steelcontent lulc localproduct
									
										distancesq_hp1 distancesq_hp2 distancesq_hp3 distancesq_hp4 distancesq_hp5
										distancesq_li1 distancesq_li2 distancesq_li3 distancesq_li4 distancesq_li5
										distancesq_wi1 distancesq_wi2 distancesq_wi3 distancesq_wi4 distancesq_wi5
										distancesq_he1 distancesq_he2 distancesq_he3 distancesq_he4 distancesq_he5
										
										ONdistancesq_hp0 ONdistancesq_hp1 ONdistancesq_hp2 ONdistancesq_hp3 ONdistancesq_hp4 ONdistancesq_hp5 ONdistancesq_hp6
										ONdistancesq_li0 ONdistancesq_li1 ONdistancesq_li2 ONdistancesq_li3 ONdistancesq_li4 ONdistancesq_li5 ONdistancesq_li6
										ONdistancesq_wi0 ONdistancesq_wi1 ONdistancesq_wi2 ONdistancesq_wi3 ONdistancesq_wi4 ONdistancesq_wi5 ONdistancesq_wi6
										ONdistancesq_he0 ONdistancesq_he1 ONdistancesq_he2 ONdistancesq_he3 ONdistancesq_he4 ONdistancesq_he5 ONdistancesq_he6
										
										CFis_hp 	CFis_li 	CFis_wi 	CFis_he 	CFis_num
										ifs_hp 		ifs_li 		ifs_wi 		ifs_he 		ifs_num
										FONis_hp 	FONis_li 	FONis_wi 	FONis_he  	FONis_num
										FONifs_hp 	FONifs_li 	FONifs_wi 	FONifs_he  	FONifs_num";										

foreach x in `1nl_inst' {;
egen m`x'=mean(`x'), by(model_group);
gen d`x'=`x'-m`x';
drop m`x';
};


			
* Index vector for market ;

egen MKTSeg=group(yema Fclass);

*** Keep the variables I need for MATLAB;

keep 	ma model_group model model_jato version_jato md body year firm  brand q  gp taxprinc  population gdppercapita price_retail_adjusted_eurobp Fclass  household 
		origin sj s0 co2 fuel
		sg
		sjg
		lsj lsjg
		dlsj
		lsjg dlsjg
		Fclass1 Fclass2 Fclass3 Fclass4 Fclass5
		lsjg_Fclass1 lsjg_Fclass2 lsjg_Fclass3 lsjg_Fclass4 lsjg_Fclass5
		dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5
		yema oldyema MKTSeg
		hp  eurokm  wi he size li nhome taxprinc  md1-md12  ma1 ma2 ma3  yema1-yema42
		dhp  deurokm  dwi dhe  dnhome  dtaxprinc  dmd*  dma*  dyema1-dyema42
		dtime1	dtime2	dtime3	 	
		dtime21	dtime22	dtime23

		dsteelcontent dlulc dlocalproduct
									
		ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
		ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
		ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
		ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
										
	    dONdistancesq_hp0 dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5 dONdistancesq_hp6
		dONdistancesq_li0 dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 dONdistancesq_li6
		dONdistancesq_wi0 dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5 dONdistancesq_wi6
		dONdistancesq_he0 dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5 dONdistancesq_he6	
		
		dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
		difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
		dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
		dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num;																			
						
*** Regression with demeaned variables;
/*
#delimit ;

ivregress 2sls dlsj	(dtaxprinc 			dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5 =
										dsteelcontent dlulc dlocalproduct
									
										ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
										ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
										ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
										ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
																		
										dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5
										dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 
										dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5
										dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5
										
										dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
										difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
										dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
										dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num)
						dhp deurokm dwi dhe dnhome
						dmd1 dmd2 dmd3 dmd4 dmd5 dmd6 dmd7 dmd8 dmd9 dmd10 dmd11 	
						dyema1-dyema41 ;*/

#delimit cr
* Form a balanced dataset
egen model2=group(model), label
sort yema Fclass model2
drop model
rename model2 model

xtset model yema
tsset model yema
tsfill, full

sort model Fclass
by model: replace Fclass=Fclass[_n-1] if missing(Fclass)


gen misid=1 if taxprinc!=.
replace misid=0 if misid==.
order misid
rename size carsize

* I created a "balanced" dataset in Stata by filling the missings the unbalancedness is ignorable (the missingness is unrelated to the
* dependent variable) so the missing data can be zeroed-out all methods for
* balanced data go through - zeroed out means replacing nan with zeros

mvencode _all, mv(0) override


order yema Fclass model
/*
#delimit ;
ivregress 2sls dlsj	(dtaxprinc 			dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5 =
										dsteelcontent dlulc dlocalproduct
									
										ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5
										ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5
										ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5
										ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5
																		
										dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5
										dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 
										dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5
										dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5
										
										dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num
										difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num
										dFONis_hp 	dFONis_li 	dFONis_wi 	dFONis_he  	dFONis_num
										dFONifs_hp 	dFONifs_li 	dFONifs_wi 	dFONifs_he  dFONifs_num)
						dhp deurokm dwi dhe dnhome
						dmd1 dmd2 dmd3 dmd4 dmd5 dmd6 dmd7 dmd8 dmd9 dmd10 dmd11
						dyema1-dyema41 ;*/
											
#delimit cr

* Very important!
sort yema Fclass model

	compress

	saveold "...\bootstrap_samples2\WTSegboot`j'", replace
	
	restore
	local j = `j' + 1
}
}

clear


local j = 1
while `j' <= 100 {
	quietly {
	preserve

	use "...\bootstrap_samples\WTSegboot`j'"
	
	outsheet using "...\bootstrap_samples\WTSegboot`j'", nolabel replace
	
	restore
	local j = `j' + 1
}
}