function [sk,skou]=ONGEVShareCalculation(theta,delta,Data)

% ONGEVShareCalculation - Ordered Logit share calculation
% Syntax:   [sk]=ONGEVShareCalculation(thetaNL,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sk    = market shares
% Subfunctions: none
%

% Author: Laura Grigolon
% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
multidumnest    = Data.multidumnest;
M               = Data.M;
Nnest           = Data.Nnest;
weightmat       = Data.weightmat;
Nsigma          = Data.Nsigma;

% TrueShare - Nested Logit share calculation using true parameter values
% Refer to excel for a layout of the procedure

alpha       = theta(1);

sigmaNest   = theta(2:Nsigma+1,1);
pho0   = theta(7,1);
pho01  = theta(7,1);
pho012 = theta(7,1);
pho123 = theta(8,1);
pho234 = theta(8,1);
pho345 = theta(8,1);
pho45  = theta(8,1);
pho5   = theta(8,1);


pho       = [pho0; pho01; pho012; pho123; pho234; pho345; pho45; pho5];
sigmaNest = [0;sigmaNest];


skInd     = NaN(prods,nmkt,10);

% loop for 10 income deciles - use loop since I have only 10 draws

for i = 1:10
    
    % Add outside good in delta
    deltaou             = zeros((prods+1),nmkt);
    
    mudel = delta + alpha .* Data.PriceInc(:,:,i);
    deltaou(2:1:end,:)  = mudel;
    deltaou             = reshape(deltaou,[(prods+1)*nmkt,1]);
    
    numer1      = exp(deltaou);
    
    % use multidimensional arrays
    numer1      = reshape(numer1,[prods+1,1,nmkt]);
    numer1      = bsxfun(@times,numer1,multidumnest);
    

    numer1    = bsxfun(@power,numer1,(1./(1-sigmaNest')));
    denom1    = nansum(numer1,1);
    qkn       = bsxfun(@rdivide,numer1,denom1);
    
    % create Z matrix
    Z = bsxfun(@power,denom1,(ones(1,Nnest)-sigmaNest')); % Z = 1 x #Nests x #Markets
    Z = repmat(Z,[M+1 1 1]);
    
    Zmat = zeros(Nnest+M, Nnest, nmkt);
    
    for j=(1:Nnest)
        Zmat(j:1:((M+1)+(j-1)),j,:) = Z(:,j,:);
    end
    
    numer2 = weightmat.*(bsxfun(@power,Zmat,1./(ones(Nnest+M,1)-pho)));
    denom2 = sum(numer2,2);
    qnBr   =  bsxfun(@rdivide, numer2, denom2);
    qnBr(isnan(qnBr))=0;
    
    numer3    = bsxfun(@power,denom2,(ones(Nnest+M,1)-pho));
    denom3    = sum(numer3);
    qBr       = bsxfun(@rdivide, numer3, denom3);
    
    partial   = bsxfun(@times,qnBr,qBr);
    partial2 = zeros(M+1,Nnest,nmkt);
    for j = 1:Nnest
        partial2(:,j,:) =  partial(Data.helpMktShare(j,:),j,:);
    end
    
    partial   = reshape(partial2,[M+1 1 Nnest nmkt]);
    partial   = permute(partial,[2 3 1 4]);
    
    qkn   = repmat(qkn,[1 M+1 1]);
    qkn   = reshape(qkn,[prods+1 Nnest M+1 nmkt]);
    
    sk   = bsxfun(@times,qkn,partial);
    sk   = nansum(sk,2);
    sk   = nansum(sk,3);
    %2D  = prods X mkt
    skou = squeeze(sk);
    % remove outside good
    skInd(:,:,i)   = skou(2:end,:);
    
end

sk = mean(skInd,3);
end