function f = gmmONGEV(theta,data)
% gmmNL - This function is used to calculate the GMM estimator for the
% Nested Logit function
%
% [f df]=gmmNL(thetaNL,BLPdata)
%
% Inputs:
%    input1 - Non linear parameters nested logit
%    input2 - Data
%
% Outputs:
%    f = gmm function value; df = gradient of f
%
% Subfunctions: delta(theta,data)
%

% Author: Laura Grigolon

%% Contraction Mapping
[d,iterat]   = delta(theta,data);
d            = reshape(d,[data.nobs 1]);
d2           = d;
d(~isfinite(d))  = 0;

%% GMM
d2(data.misidhelp==0)=[];
if max(isnan(d2)) == 1 || iterat>3000 || isreal(d2)==0
    f  = 1e8;
else
    % Use relationship between linear and non-linear parameters from step 4,
    % resulting from the FOC's
    beta    = data.invxzwzx*(data.xzwz*d);
    % error term
    csi     = d - data.X*beta;
    f       = csi'*data.Z*data.W*data.Z'*csi;
end
end