%Clear and set Diary
clear;
load WTSeg.mat;
tic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1 Initial setting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nmkt        = 42;                        % number of markets
prods       = size(brand,1)/nmkt;          % number of products
nobs        = nmkt*prods;                % number of observations
Nsigma      = 5;                         % number of nests without outside good
Nnest       = 5;

dumseg           = dummyvar(Fclass);
multidumseg      = reshape(dumseg',[Nnest,prods,nmkt]);
Data.multidumseg = permute(multidumseg,[2 1 3]);

%% Add outside good as the inferior quality good: get rid of this part if you do not want the outside good
nestou              = zeros((prods+1),nmkt);
nestr               = reshape(Fclass,[prods nmkt]);
nestou(2:1:end,:)   = nestr;
nestou              = reshape(nestou,[(prods+1)*nmkt,1]);
nestou              = nestou+1;     % this is to get positive matrix indices;

dumnest       = dummyvar(nestou);
multidumnest  = reshape(dumnest',[Nnest+1,(prods+1),nmkt]);
multidumnest  = permute(multidumnest,[2 1 3]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2. Estimation with Homogeneity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Matrix of exogenous variables
Xexo                  = [dhp, deurokm, dwi, dhe, dnhome, ...
                        dmd1, dmd2, dmd3, dmd4, dmd5, dmd6, dmd7, dmd8, dmd9, dmd10, dmd11,...
						dyema2, dyema3, dyema4, dyema5, dyema6, dyema7, dyema8, dyema9, dyema10,...
                        dyema11, dyema12, dyema13, dyema14, dyema15, dyema16, dyema17, dyema18, dyema19, dyema20,...
                        dyema21, dyema22, dyema23, dyema24, dyema25, dyema26, dyema27, dyema28, dyema29, dyema30,...
                        dyema31, dyema32, dyema33, dyema34, dyema35, dyema36, dyema37, dyema38, dyema39, dyema40,...
                        dyema41, dyema42];
                    
% Matrix of instruments 
Zinst =[								dsteelcontent dlulc dlocalproduct...
										ddistancesq_hp1 ddistancesq_hp2 ddistancesq_hp3 ddistancesq_hp4 ddistancesq_hp5...
										ddistancesq_li1 ddistancesq_li2 ddistancesq_li3 ddistancesq_li4 ddistancesq_li5...
										ddistancesq_wi1 ddistancesq_wi2 ddistancesq_wi3 ddistancesq_wi4 ddistancesq_wi5...
										ddistancesq_he1 ddistancesq_he2 ddistancesq_he3 ddistancesq_he4 ddistancesq_he5...	
										dONdistancesq_hp1 dONdistancesq_hp2 dONdistancesq_hp3 dONdistancesq_hp4 dONdistancesq_hp5...
										dONdistancesq_li1 dONdistancesq_li2 dONdistancesq_li3 dONdistancesq_li4 dONdistancesq_li5 ...
										dONdistancesq_wi1 dONdistancesq_wi2 dONdistancesq_wi3 dONdistancesq_wi4 dONdistancesq_wi5...
										dONdistancesq_he1 dONdistancesq_he2 dONdistancesq_he3 dONdistancesq_he4 dONdistancesq_he5...
                                        dCFis_hp 	dCFis_li 	dCFis_wi 	dCFis_he 	dCFis_num...
                                        difs_hp 	difs_li 	difs_wi 	difs_he 	difs_num...
                                        dFONis_hp   dFONis_li   dFONis_wi   dFONis_he   dFONis_num...
										dFONifs_hp  dFONifs_li  dFONifs_wi  dFONifs_he  dFONifs_num];

 % Rescale to improve optmization
 
% Give the number of explanatory variables which enter linearly
nlc = size(Xexo,2);

%DEFINE LABELS
totlabels={'hp' 'eurokm' 'width' 'height' 'nhome'...
            'md1' 'md2' 'md3' 'md4' 'md5' 'md6' 'md7' 'md8' 'md9' 'md10' 'md11'...
            'yema2' 'yema3' 'yema4' 'yema5'	'yema6'	'yema7'	'yema8'	'yema9'	'yema10'...
            'yema11'	'yema12'	'yema13'	'yema14'	'yema15'	'yema16'	'yema17'	'yema18'	'yema19' 'yema20'...
            'yema21'	'yema22'	'yema23'	'yema24'	'yema25'	'yema26'	'yema27'	'yema28'	'yema29'	'yema30'...
            'yema31'	'yema32'	'yema33'	'yema34'	'yema35'	'yema36'	'yema37'	'yema38'	'yema39'	'yema40'...
            'yema41'	'yema42' ...		
            'princ' 'lsjg_cla1'	'lsjg_cla2'	'lsjg_cla3'	'lsjg_cla4'	'lsjg_cla5'};

% create the X and Z matrix out of the exogenous, endogenous and
% instrumental pieces

X = [Xexo dtaxprinc dlsjg_Fclass1 dlsjg_Fclass2 dlsjg_Fclass3 dlsjg_Fclass4 dlsjg_Fclass5];
Z = [Xexo Zinst];
Y = log(sj)-log(s0);

Data.obsshare = sj;

% STAGE I: Initial weighting matrix
norm    = mean(mean(Z'*Z));
W       = ((Z'*Z)/norm)\eye(size(Z,2))/norm;
Btsls   = ((X'*Z*W*Z'*X)\eye(size(X,2)))*(X'*Z*W*Z'*dlsj);

SigmaNest1 = Btsls(nlc+2:end);
DeltaStart = Y - (bsxfun(@times,lsjg,dumseg)) * SigmaNest1;

%Function Value at the Minimum:
functvalueNL1   = (dlsj-X*Btsls)'*Z*W*Z'*(dlsj-X*Btsls);
csi0            = dlsj-X*Btsls;

% STAGE II: Optimal weighting matrix 
csi0Z   = bsxfun(@times,csi0,Z);
norm    = mean(mean(csi0Z'*csi0Z));
W1      = ((csi0Z'*csi0Z)/norm)\eye(size(csi0Z,2))/norm;
Bgmm    = ((X'*Z*W1*Z'*X)\eye(size(X,2)))*(X'*Z*W1*Z'*dlsj);

%Function Value at the Minimum:
functvalueNL2  = (dlsj-X*Bgmm)'*Z*W1*Z'*(dlsj-X*Bgmm);
csi1            = dlsj-X*Bgmm;

% Calculation of the standard errors (beta 2SLS)
part1   = X'*Z;
vv1     = inv(part1*W*part1');
% Fixed effects compensation of the error term
WTvariable = model_group;
xavt       = zeros(size(X,1),size(X,2));
davt       = zeros(size(Y,1),1);
codeteller = min(WTvariable);
while codeteller <= max(WTvariable)
    codeselection = WTvariable==codeteller;
    if max(codeselection)>0
        dsum    = nansum(codeselection.*Y)/sum(codeselection);
        davt    = davt+codeselection.*dsum;
        prodx   = bsxfun(@times,codeselection,X);
        xsum    = sum(prodx)/sum(codeselection);
        xavt    = xavt+codeselection*xsum;
    end
    codeteller  = codeteller+1;
end

muest=davt-xavt*Btsls;

ksi     = Y-X*Btsls-muest;
ksi(~any(~isnan(ksi), 2),:)=[];
dgf     = size(ksi,1)-size(X,2);
est     = ksi'*ksi/dgf;
sthetaNL = sqrt(est*diag(vv1));
tthetaNL = Btsls./sthetaNL;

% Calculation of the standard errors (beta GMM) 
vv1         = inv(part1*W1*part1');
sthetaNL2   = sqrt(diag(vv1));
tthetaNL2   = Bgmm./sthetaNL2;

%Print Results
title1      = {'var','2SLS','st.err.','tvalue','GMM','st.err.','tvalue'};
resultNestedLogit   = [Btsls sthetaNL tthetaNL Bgmm sthetaNL2 tthetaNL2];
disp('Results of nested logit demand estimation')
disp([title1;totlabels.',num2cell(resultNestedLogit)])

%write nested logit results to xlsx
xlswrite('results.xlsx',totlabels.','Nested Logit','A2')
xlswrite('results.xlsx',resultNestedLogit,'Nested Logit','B2')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3. RCNL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Add income distribution

% this ad hoc

% France
IncomeDecileFR = [3.5,	5.099999905,	6.199999809,	7,	7.900000095,	8.800000191,	10,	11.5,	14,	25.89999962];
% Germany
IncomeDecileDE = [3.200000048,	5.199999809,	6.300000191,	7.300000191,	8.300000191,	9.399999619,	10.69999981,	12.19999981	14.5,	22.89999962];
% Italy
IncomeDecileIT = [2.299999952,	4.699999809,	6,	7.199999809,	8.199999809,	9.5,	10.80000019,	12.39999962,	14.89999962,	24.20000076];

IncomeMaFR      = bsxfun(@times,IncomeDecileFR,ma(1:4158));
IncomeMaDE      = bsxfun(@times,IncomeDecileDE,ma(4159:8316)==2);
IncomeMaIT      = bsxfun(@times,IncomeDecileIT,ma(8317:end)==3);

Incomedraw  = [IncomeMaFR ; IncomeMaDE ; IncomeMaIT]./10;
% Incomedraw    = ones(prods*nmkt,10); % This line changes it to the homogeneous case

Data.PriceInc = bsxfun(@rdivide,taxprinc,Incomedraw);
Data.PriceInc = reshape(Data.PriceInc, [prods nmkt 10]);


% Create the X and Z matrix of exogenous, endogenous elements and instruments
misidhelp        = reshape(misid,[prods nmkt]);
X                = Xexo ;

xzwz             = X'*Z*W*Z';

xzwzx            = xzwz * X;
locnorm          = mean(mean(xzwzx));
invxzwzx         = (xzwzx/locnorm)\eye(size(xzwzx,2))/locnorm;

Data.misidhelp    = misidhelp;
Data.X            = X;
Data.Z            = Z;
Data.W            = W;
Data.xzwz         = xzwz;
Data.invxzwzx     = inv(xzwzx);
Data.logobsshare  = reshape(log(sj),[prods nmkt]);
Data.nobs         = nobs;
Data.prods        = prods;
Data.nmkt         = nmkt;
Data.lsjg         = lsjg;
Data.Nsigma       = Nsigma;

%     OPTIMIZATION
%     Options:see Knitro Option File KnitroBLP.opt
%     Pass Data into gmm using anonymous functions

theta0 = Btsls(nlc+1:end); % starting values

% OPTIMIZATION
% Options:see Knitro Option File KnitroBLP.opt
% Pass Data into gmm using anonymous functions
% DeltaStart(isnan(DeltaStart))=0;

DeltaStart  = Y - (bsxfun(@times,lsjg,dumseg)) * SigmaNest1 - Btsls(nlc+1).*taxprinc;
Data.delta0 = reshape(DeltaStart, [prods nmkt]);
% [sj,sij,sijg,numer1,denom1,denom2] = NLShareCalculation(theta0,Data.delta0,Data);

angmmNL  = @(theta0)gmmNL(theta0,Data);

options = optimset( 'Display','iter',...
    'GradObj','off','TolCon',1E-6,...
    'TolFun',1E-6,'TolX',1E-6,...
    'Hessian', 'off','DerivativeCheck','off','FinDiffType','central');

x_L     = [-10;zeros(numel(theta0)-1,1)];       % lower bound (for nesting parameters to avoid numerical problems when sigma/pho gets negative)
x_U     = [10;ones(numel(theta0)-1,1)*0.975];         % upper bound (for nesting parameters to avoid numerical problems when sigma/pho gets close to 1)


% Constraint on magnitude

[thetaNL, FctValNL, exitflagNL,outputNL,lambda,gradient, hessian] = ...
    fmincon(angmmNL,theta0,[],[],[],[],x_L,x_U,[],options);


deltNL       = deltaNL(thetaNL,Data);
Data.delta0  = reshape(deltNL, [prods nmkt]);

deltNL(~isfinite(deltNL))  = 0;
deltNL       = reshape(deltNL,[nobs 1]);
betaNL       = invxzwzx*(xzwz*deltNL);
betathetaNL  = [betaNL ; thetaNL];

derDeltaTheta   = numgradSE(@deltaNL,thetaNL,Data);

XstErr = X;
XstErr(misid==0,:)=[];

ZstErr = Z;
ZstErr(misid==0,:)=[];

deltNL(misid==0,:)=[];

derksi=[-XstErr  derDeltaTheta]'*ZstErr;
vv=inv(derksi*W*derksi');

% Fixed effects compensation of the error term
WTvariable = model_group;
WTvariable(misid==0,:)=[];
xavt       = zeros(size(XstErr,1),size(XstErr,2));
davt       = zeros(size(deltNL,1),1);
codeteller = min(WTvariable);
while codeteller <= max(WTvariable)
    codeselection = WTvariable==codeteller;
    if max(codeselection)>0
        dsum    = nansum(codeselection.*deltNL)/sum(codeselection);
        davt    = davt+codeselection.*dsum;
        prodx   = bsxfun(@times,codeselection,XstErr);
        xsum    = sum(prodx)/sum(codeselection);
        xavt    = xavt+codeselection*xsum;
    end
    codeteller  = codeteller+1;
end

muest=davt-xavt*betaNL;

ksi     = deltNL-XstErr*betaNL-muest;
dgf     = size(XstErr,1)-size(XstErr,2);
est     = ksi'*ksi/dgf;
stheta  = sqrt(est*diag(vv));
ttheta  = betathetaNL./stheta;

%% Second stage: heterosk. var-covar

% Stage II optimal weighting matrix (update W)

% Add back the zeros in ksi
ksi0        = zeros(nobs,1);
indexValue  = find(misid);

for i = 1:length(ksi)
    ksi0(indexValue(i)) = ksi(i);
end

ksiZ   = bsxfun(@times,ksi0,Z);
norm    = mean(mean(ksiZ'*ksiZ));
Wgmm    = ((ksiZ'*ksiZ)/norm)\eye(size(ksiZ,2))/norm;

xzwz             = X'*Z*Wgmm*Z';
xzwzx            = xzwz * X;
locnorm          = mean(mean(xzwzx));
invxzwzx         = (xzwzx/locnorm)\eye(size(xzwzx,2))/locnorm;

Data.X            = X;
Data.Z            = Z;
Data.W            = Wgmm;
Data.xzwz         = xzwz;
Data.invxzwzx     = inv(xzwzx);
Data.logobsshare  = reshape(log(sj),[prods nmkt]);

%     OPTIMIZATION
%     Options:see Knitro Option File KnitroBLP.opt
%     Pass Data into gmm using anonymous functions
% theta0 = zeros(Nnest+Nnest+M+1,1) ;     % starting value

theta0 = thetaNL;

% OPTIMIZATION
% Options:see Knitro Option File KnitroBLP.opt
% Pass Data into gmm using anonymous functions
% DeltaStart(isnan(DeltaStart))=0;
deltNL       = deltaNL(thetaNL,Data);
Data.delta0  = reshape(deltNL, [prods nmkt]);

angmmNL  = @(theta0)gmmNL(theta0,Data);

options = optimset( 'Display','iter',...
    'GradObj','off','TolCon',1E-6,...
    'TolFun',1E-6,'TolX',1E-6,...
    'Hessian', 'off','DerivativeCheck','off','FinDiffType','central');

[thetaNLgmm, FctValNLgmm, exitflagNLgmm,outputNLgmm,lambda,gradient, hessian] = ...
    fmincon(angmmNL,theta0, [], [], [], [], x_L, x_U, [],options);

deltNLgmm       = deltaNL(thetaNLgmm,Data);
Data.deltstart  = deltNLgmm;

deltNLgmm       = reshape(deltNLgmm,[nobs 1]);
deltNLgmm(isnan(deltNLgmm))  = 0;
betaNLgmm       = invxzwzx*(xzwz*deltNLgmm);
betathetaNLgmm  = [betaNLgmm ; thetaNLgmm];

derDeltaTheta      = numgradSE(@deltaNL,thetaNLgmm,Data);

derksi      = [-XstErr  derDeltaTheta];
dermom      = ZstErr' * derksi;
vvgmm       = inv(dermom'*Wgmm*dermom);
sthetagmm   = sqrt(diag(vvgmm));
tthetagmm   = betathetaNLgmm ./sthetagmm;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 11. Report and Wrap UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print And Save The Logit Estimation
% Print Results
title1={'var','First Stage','st.err.','tvalue','GMM','st.err.','tvalue'};
resultNL=[betathetaNL stheta ttheta betathetaNLgmm sthetagmm tthetagmm];
disp('Results ')
disp('Results NL')
xexolabels = totlabels ;
disp([title1;xexolabels.',num2cell(resultNL)])


% Print And Save the NL Estimation

disp(['Function Value Nested Logit 2SLS' num2str(functvalueNL1)])
disp(['Function Value Nested Logit GMM' num2str(functvalueNL2)])

disp(['Function Value RCNL 2SLS' num2str(FctValNL)])
disp(['Function Value RCNL GMM' num2str(FctValNLgmm)])

xlswrite('results.xlsx',resultNL,'resultNL','B2')
xlswrite('results.xlsx',xexolabels.','resultNL','A2')

timetorun = toc/60;          %Stops the timer and gives minutes to compute
date = datestr(clock);

% Save data for counterfactuals
Data.household      = household;
Data.firm           = firm;
Data.gdppercapita   = gdppercapita;
Data.price          = price_retail_adjusted_eurobp;
Data.sj             = sj;
Data.q              = q;
Data.misid          = misid;
Data.yema           = yema;
Data.model          = model;
Data.model_group    = model_group;
Data.multidumseg    = multidumseg;
Data.Fclass         = Fclass;
Data.co2            = co2;

Result = v2struct(Btsls, sthetaNL, tthetaNL, Bgmm, sthetaNL2, tthetaNL2, ...
                  betathetaNL, stheta, ttheta, betathetaNLgmm, sthetagmm, tthetagmm, ... 
                  functvalueNL1, functvalueNL2, FctValNL, FctValNLgmm,...
                  outputNL,outputNLgmm, vv, vvgmm);

Result.Y             = Y;
Result.deltaNL    = deltNL;
Result.deltaNLgmm = deltNLgmm;             
              
save ResultEstimation Data Result