function [sj,sij,sijg,numer1,denom1,denom2] = NLShareCalculation(theta,delta,Data)

% NLShareCalculation - Nested Logit share calculation
% Syntax:   [sij sijg sj sjg s0 numer1 denom1 numer2 denom2]=NLShareCalculation(thetaNL,delta,Data)
%
% Input:
%    input1 - Non linear parameters
%    input2 - Data
%
% Output:
%    sij    = individual market shares
%    sijg   = individual conditional share of choosing good j from group g
%    sj     = market shares
%    sjg    = conditional share of choosing good j from group g 
%    s0     = outside good
%    numer1 = numerator1
%    denom1 = denominator1
%    numer2 = numerator 2
%    denom2 = denominator 2
%
% Subfunctions: none
%

% Author: Laura Grigolon
% August 2012;

% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
Nsigma          = Data.Nsigma;
nincome         = 10;
multidumseg     = repmat(Data.multidumseg,[1,1,1,nincome]);

alpha    = theta(1);
Sigseg   = theta(2:Nsigma+1,1);

% Market share calculation

mudel           = delta + alpha .* Data.PriceInc;
numer1          = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt,nincome]);
numer1          = (numer1.*multidumseg).^(1./(1-Sigseg'));
denom1          = nansum(numer1,1);

numer2          = denom1.^(1-Sigseg');
denom2          = 1 + nansum(numer2,2);                         % sum across segments within a market (second dimension)


sijg              = bsxfun(@rdivide,numer1,denom1);

sig               = bsxfun(@rdivide,numer2,denom2);

sij               = bsxfun(@times,sijg,sig);

sj                = nanmean(sij,4);
sj                = nansum(sj,2);
sj                = squeeze(sj);

sj(sj==0)         = NaN;

end