function [f] = gmmNL(thetaNL,data)
% gmmNL - This function is used to calculate the GMM estimator for the
% Nested Logit function
% 
% [f df]=gmmNL(thetaNL,BLPdata)
%
% Inputs:
%    input1 - Non linear parameters nested logit
%    input2 - Data
%
% Outputs:
%    f = gmm function value; df = gradient of f 
%
% Subfunctions: deltaLogit(theta,BLPdata) ; jacobLogit
%

% Author: Laura Grigolon and Frank Verboven
% August 2012;

%% Contraction Mapping
[d,iterat]   = deltaNL(thetaNL,data);
d            = reshape(d,[data.nobs 1]);
d2           = d;
d(~isfinite(d))  = 0;

%% GMM
d2(data.misidhelp==0)=[];
if max(isnan(d2)) == 1 || iterat>3000 || isreal(d2)==0
    f  = 1e8;
else
    % Use relationship between linear and non-linear parameters from step 4,
    % resulting from the FOC's
    beta    = data.invxzwzx*(data.xzwz*d);
    % error term
    csi     = d - data.X*beta;
    f       = csi'*data.Z*data.W*data.Z'*csi;
end

%% Gradient
% if max(isnan(d)) == 1 || iterat>2500 || isreal(d)==0    
%     % isreal(d)==0 can happen because of the negative weights - this is a problem of quadrature methods
%     df=1e10;
% else
%         temp    = jacobNL(thetaNL,d,data);
%         df      = 2*temp'*data.Z*data.W*data.Z'*csi;
% end

end

