function [ElaNLogit]=ElastNestedLogit(alpha,thetaNL,delta,characteristic,Data)

% ElastNestedLogit - Calculate price elasticities for Nested Logit models
% Syntax:  f=ElastLogit(alpha,theta,delta,Data)
%
% Inputs:
%    input1 - alpha     = price coefficient
%    input2 - thetaNL   = random coefficient
%    input1 - delta     = mean value
%    input2 - Data
%
% Outputs:
%    prods x prods elasticity matrix
%
% Subfunctions: NLShareCalculation 
%
% Author: Laura Grigolon
% August 2012;

% Unpack
prods            = Data.prods;
nmkt             = Data.nmkt;

% Market Shares
[sij,sijg,sj,~,~, ~,~,~,~] = NLShareCalculation(thetaNL,delta,Data);
Sigseg          = thetaNL(1:Data.Nsigma);

% Derivative wrt delta
part1            = sum(bsxfun(@times,(1./(1-Sigseg')), sij),2);
part1            = bsxfun(@times,part1,eye(prods));          % diagonal in multiple dimensions

sijtransp        = permute(sij,[2 1 3 4]);
sijg2            = bsxfun(@times,Sigseg'./(1-Sigseg'),sijg);         
part2            = multiprod(sijg2,sijtransp);

part3            = multiprod(sum(sij,2),sum(sijtransp,1));

derShareDelta   = (part1 - part2 - part3);

PriceOverShare   = multiprod((1./(sum(sj,2))),permute(characteristic,[2 1 3]));
ElaNLogit = bsxfun(@times,derShareDelta*(alpha./mean(Data.gdppercapita)),PriceOverShare);

end
