function [ElaONGEV, Diversion]=ElastONGEV(alpha,theta,delta,price,Data)

% ElastONGEV - Calculate price elasticities for ONGEV models
% Syntax:  f=ElastLogit(alpha,theta,delta,Data)
%
% Inputs:
%    input1 - alpha     = price coefficient
%    input2 - thetaNL   = random coefficients/nesting parameters
%    input1 - delta     = mean value
%    input2 - Data
%
% Outputs:
%    prods x prods elasticity matrix
%
% Subfunctions: NLShareCalculation 
%
% Author: Laura Grigolon
% August 2015;

% Unpack
income              = Data.gdppercapita;
Data.alphaResults   = alpha;
Data.SigmaResults   = theta(1:Data.Nsigma);
Data.PhoResults     = theta(Data.Nsigma+1:end);

sj               = ONGEVShareDelta(delta,Data);
sj(sj==0)           =[];
income(income==0)   =[];
price(price==0)     =[];

derShareDelta   = numgrad(@ONGEVShareDelta,delta,Data);

PriceOverShare  = (1./sj)*price';
ElaONGEV        = bsxfun(@times,(derShareDelta*(alpha./mean(income))),PriceOverShare);

OwnDeriv        = diagnd(derShareDelta);
Diversion       = bsxfun(@times,(derShareDelta),(1./OwnDeriv));

end