 %% 0. Upload data
clear;

load ResultEstimation.mat;

%% Select one year/market combination: Germany 2011. Yema=28

Identifier = 28;

Data.misidhelp = Data.misidhelp(:,Identifier);
Data.co2 = Data.co2(Data.yema==Identifier,:);
Data.co2(Data.misidhelp==0) = [];

Data.weightmat      = Data.weightmat(:,:,Identifier);
Data.logobsshare    = Data.logobsshare(:,Identifier);
Data.logobsshare(Data.misidhelp==0) = [];

Data.prods = sum(Data.misidhelp); % I do not need a balanced panel across markets here, so I delete missings
Data.nmkt  = 1;
Data.nobs  = Data.prods*Data.nmkt;

Data.multidumnest   = Data.multidumnest(:,:,Identifier);
misidhelp2          = [1;Data.misidhelp]; % take outside good into account
Data.multidumnest(misidhelp2==0,:,:) = [];

Data.lsjg       = Data.lsjg(Data.yema==Identifier,:);
Data.lsjg(Data.misidhelp==0) = [];

Data.multidumseg    = Data.multidumseg(:,:,Identifier);
Data.multidumseg(Data.misidhelp==0,:,:) = [];

Data.Fclass      = Data.Fclass(Data.yema==Identifier,:);
Data.Fclass(Data.misidhelp==0) = [];

Data.model          = Data.model(Data.yema==Identifier,:);
Data.model(Data.misidhelp==0) = [];

Data.model_group    = Data.model_group(Data.yema==Identifier,:);
Data.model_group(Data.misidhelp==0) = [];

Data.household      = Data.household(Data.yema==Identifier,:);
Data.household(Data.misidhelp==0) = [];

Data.firm           = Data.firm(Data.yema==Identifier,:);
Data.firm(Data.misidhelp==0) = [];

Data.gdppercapita   = Data.gdppercapita(Data.yema==Identifier,:);
Data.gdppercapita(Data.misidhelp==0) = [];

Data.price          = Data.price(Data.yema==Identifier,:);
Data.price(Data.misidhelp==0) = [];

Data.sj             = Data.sj(Data.yema==Identifier,:);
Data.sj(Data.misidhelp==0) = [];

Data.q              = Data.q(Data.yema==Identifier,:);
Data.q(Data.misidhelp==0) = [];

Data.Y              = Result.Y(Data.yema==Identifier,:);
Data.Y(Data.misidhelp==0) = [];

Data.Nsigma = 5;
Data.Npho   = 2;

multiPrice      = bsxfun(@times,Data.price,Data.multidumseg);

% Define variables that will be used for NL and ONGEV

firmdum         = dummyvar(Data.firm);
Data.ownership  = firmdum*firmdum';

%% 1. Nested Logit

% Product elasticity
alpha      = Result.Bgmm(end-Data.Nsigma);
Data.alpha = alpha;

% Y = ln(sj) - ln(s0)
SigmaGMM        = Result.Bgmm(end-Data.Nsigma+1:end);
dumseg          = dummyvar(Data.Fclass);

Data.deltaNL = Data.Y - (bsxfun(@times,Data.lsjg,dumseg)) * SigmaGMM;

thetaNL = [SigmaGMM;zeros(Data.Npho,1)];
[ElaNL, DiversionNL] = ElastONGEV(alpha,thetaNL,Data.deltaNL,Data.price,Data);


[ElaNL2] = ElastNestedLogit(alpha,thetaNL,Data.deltaNL,Data.price,Data);

[~,OneOwnElastNL,~,OneCrossElastSameSegNL,~,OneCrossElastDiffSegNL,...
    ~,~,~,~] = sumElast(ElaNL,DiversionNL,Data);

% Group elasticity

% Delta exogenous wrt price/income
Data.deltaEX    = Data.deltaNL - (alpha ./mean(Data.gdppercapita)) * Data.price;
Fclass          = [0;Data.Fclass] + 1; % Fclass including outside good

Data.theta      = thetaNL;

[test,~] = ONGEVSharePrice(Data.price,Data);

% Demand before price increase
taboutElastNL = zeros(Data.Nsigma,Data.Nsigma+1);
% Cross price elasticity neighboring same seg: mkt share after price increase 1%
for j = 1:Data.Nsigma
    % price increase per segment in all markets
    lambda          = 0.01;
    pricetemp       = multiPrice(:,j) .* (1+lambda);
    multiPricetemp  = multiPrice;
    
    multiPricetemp(:,j) = pricetemp;
    
    pricePOST       = sum(multiPricetemp,2);
    [~,sj]          = ONGEVSharePrice(Data.price,Data);
    [~,skNEW]       = ONGEVSharePrice(pricePOST,Data);
    for jj = 1:Data.Nsigma+1
        demand0                 = sum(sj.*(jj==Fclass));      % Demand before price increase
        demand1                 = sum(skNEW.*(jj==Fclass));   % Demand after price increase
        taboutElastNL((j-1)+1,jj)     = 100*((demand1-demand0)./ demand0);
    end
end

%% 1. ONGEV

% Product elasticity
alpha               = Result.betathetaONGEVgmm(58);
Data.alpha          = alpha;
thetaONGEV          = Result.betathetaONGEVgmm(59:65);
Data.SigmaResults   = thetaONGEV(1:Data.Nsigma);
Data.PhoResults     = thetaONGEV(Data.Nsigma+1:end);
% I also need to recalculate delta (contraction mapping)

delta0     = Data.Y;

save mvalold delta0
Data.deltONGEV  = delta(thetaONGEV,Data);

% test = ONGEVShareCalculation(thetaONGEV,Data.deltONGEV,Data);

[ElaONGEV, DiversionONGEV] = ElastONGEV(alpha,thetaONGEV,Data.deltONGEV,Data.price,Data);

[~,OneOwnElastONGEV,~,OneCrossElastSameSegONGEV,~,OneCrossElastDiffSegONGEV,...
    ~,~,~,~] = sumElast(ElaONGEV,DiversionONGEV,Data);

% Group elasticity

% Delta exogenous wrt price/income
Data.deltaEX     = Data.deltONGEV - (alpha ./mean(Data.gdppercapita)) * Data.price;

% Demand before price increase
taboutElastONGEV = zeros(Data.Nsigma,Data.Nsigma+1);
% Cross price elasticity neighboring same seg: mkt share after price increase 1%
for j = 1:Data.Nsigma
    % price increase per segment in all markets
    lambda          = 0.01;
    pricetemp       = multiPrice(:,j) .* (1+lambda);
    multiPricetemp  = multiPrice;
    
    multiPricetemp(:,j) = pricetemp;
    
    pricePOST       = sum(multiPricetemp,2);
    Data.theta      = thetaONGEV;
    [~,sj]          = ONGEVSharePrice(Data.price,Data);
    [~,skNEW]       = ONGEVSharePrice(pricePOST,Data);
    for jj = 1:Data.Nsigma+1
        demand0                 = sum(sj.*(jj==Fclass));      % Demand before price increase
        demand1                 = sum(skNEW.*(jj==Fclass));   % Demand after price increase
        taboutElastONGEV((j-1)+1,jj)     = 100*((demand1-demand0)./ demand0);
    end
end