%% 0. Upload data
clear;

load ResultEstimation.mat;

%% Select one year/market combination: Germany 2011. Yema=28

Identifier = 28;

Data.misidhelp = Data.misidhelp(:,Identifier);

Data.co2 = Data.co2(Data.yema==Identifier,:);
Data.co2(Data.misidhelp==0) = [];

Data.weightmat      = Data.weightmat(:,:,Identifier);
Data.logobsshare    = Data.logobsshare(:,Identifier);
Data.logobsshare(Data.misidhelp==0) = [];

Data.prods = sum(Data.misidhelp); % I do not need a balanced panel across markets here, so I delete missings
Data.nmkt  = 1;
Data.nobs  = Data.prods*Data.nmkt;

Data.multidumnest   = Data.multidumnest(:,:,Identifier);
misidhelp2          = [1;Data.misidhelp]; % take outside good into account
Data.multidumnest(misidhelp2==0,:,:) = [];

Data.lsjg       = Data.lsjg(Data.yema==Identifier,:);
Data.lsjg(Data.misidhelp==0) = [];

Data.multidumseg    = Data.multidumseg(:,:,Identifier);
Data.multidumseg(Data.misidhelp==0,:,:) = [];

Data.Fclass      = Data.Fclass(Data.yema==Identifier,:);
Data.Fclass(Data.misidhelp==0) = [];

Data.model          = Data.model(Data.yema==Identifier,:);
Data.model(Data.misidhelp==0) = [];

Data.model_group    = Data.model_group(Data.yema==Identifier,:);
Data.model_group(Data.misidhelp==0) = [];

Data.household      = Data.household(Data.yema==Identifier,:);
Data.household(Data.misidhelp==0) = [];

Data.firm           = Data.firm(Data.yema==Identifier,:);
Data.firm(Data.misidhelp==0) = [];

Data.gdppercapita   = Data.gdppercapita(Data.yema==Identifier,:);
Data.gdppercapita(Data.misidhelp==0) = [];

Data.price          = Data.price(Data.yema==Identifier,:);
Data.price(Data.misidhelp==0) = [];

Data.sj             = Data.sj(Data.yema==Identifier,:);
Data.sj(Data.misidhelp==0) = [];

Data.q              = Data.q(Data.yema==Identifier,:);
Data.q(Data.misidhelp==0) = [];

Data.Y              = Result.Y(Data.yema==Identifier,:);
Data.Y(Data.misidhelp==0) = [];

Data.Nsigma = 5;
Data.Npho   = 2;

multiPrice      = bsxfun(@times,Data.price,Data.multidumseg);

% Define variables that will be used for NL and ONGEV

firmdum         = dummyvar(Data.firm);
Data.ownership  = firmdum*firmdum';

%% Before the exit

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Remove luxury compact cars

AudiA1    = Data.model(:,1)==6;
MiniHatch = Data.model(:,1)==158;
MiniWagon = Data.model(:,1)==159;
Abarth    = Data.model(:,1)==54;
 
AllRemove = AudiA1|MiniHatch|MiniWagon|Abarth;


% Automatize the removal of columns by applying it to all fiels 

Select = Data;

Select.firm(AllRemove,:)        =[];
Select.price(AllRemove,:)       =[];
Select.Y(AllRemove,:)           =[];
Select.Fclass(AllRemove,:)      =[];
Select.lsjg(AllRemove,:)        =[];
Select.prods                    =size(Select.firm,1);
Select.multidumnest(AllRemove,:)=[];
Select.gdppercapita(AllRemove,:)=[];
Select.model(AllRemove,:)       =[];

Select.firmdum       = dummyvar(Select.firm);
Select.ownership     = Select.firmdum*Select.firmdum';

Data.dumseg         = dummyvar(Data.Fclass);
Select.firmdum      = dummyvar(Select.firm);
Select.ownership    = Select.firmdum*Select.firmdum';

%% 1. Nested Logit

alpha           = Result.Bgmm(end-Data.Nsigma);
Data.alpha      = alpha;

% Y = ln(sj) - ln(s0)

Sigma           = Result.Bgmm(end-Data.Nsigma+1:end);
Select.dumseg   = dummyvar(Select.Fclass);

Select.deltaNL  = Select.Y - (bsxfun(@times,Select.lsjg,Select.dumseg)) * Sigma;

thetaNL             = [Sigma;zeros(Data.Npho,1)];
Data.deltaNL        = Data.Y - (bsxfun(@times,Data.lsjg,Data.dumseg)) * Sigma;
Data.SigmaResults   = thetaNL(1:Data.Nsigma);
Data.PhoResults     = thetaNL(Data.Nsigma+1:end);
% Before exit
[~,sjBEFORE]         = ONGEVShareDelta(Data.deltaNL,Data);

% After exit
Select.SigmaResults   = thetaNL(1:Data.Nsigma);
Select.PhoResults     = thetaNL(Data.Nsigma+1:end);
Select.alpha          =  alpha;

Data.theta            = [Select.SigmaResults;Select.PhoResults];

Data.delta      = Data.deltaNL - (Data.alpha./Data.gdppercapita) .* Data.price;
derSharePrice   = deriv(@ONGEVSharePrice,Data.price,Data);

elastNL             = bsxfun(@rdivide,Data.price',Data.sj) .* derSharePrice;

deltmin         = Data.ownership .* derSharePrice';
mcost           = Data.price + (deltmin\Data.sj);
Data.mcost      = mcost;

% Remove the product and recalculate prices
Select.mcost                = Data.mcost;
Select.mcost(AllRemove,:)   = [];
Select.delta    = Select.deltaNL - (Select.alpha./Select.gdppercapita) .* Select.price;

newprice        = fsolve1(@FOC,Select.price,Select);

% Shares under new price equilibrium

[skAFTER,skouAFTER] = ONGEVSharePrice(newprice,Select);

Select.sj = Data.sj;
Select.sj(AllRemove,:) = [];

SharebyNest0 = [sjBEFORE(1);zeros(Data.Nsigma,1)];
SharebyNest1 = [skouAFTER(1);zeros(Data.Nsigma,1)];
DiversionNL  = [(skouAFTER(1) - (1 - sum(Data.sj)))./(sum(AllRemove.*Data.sj))*100;zeros(Data.Nsigma,1)];

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1) = sum(Select.sj(Select.Fclass==j));
    SharebyNest1(j+1) = sum(skAFTER(Select.Fclass==j));
    DiversionNL(j+1)  = (- (sum(skAFTER(Select.Fclass==j)) - sum(Select.sj(Select.Fclass==j)))./(-sum(AllRemove.*Data.sj)))*100;

end    

% Diversion ratio = 
% (Sum quantities after removal excl. removed) - (Sum quantities before removal excl. removed)./ sum quantitiy removed product before
% See IJIO The impact of online sales on consumers and firms page 61

% Percent change
MktShareChangeNL = ((SharebyNest1./SharebyNest0) - 1)*100;

%% 1. ONGEV
% Product elasticity
alpha               = Result.betathetaONGEVgmm(58);
Data.alpha          = alpha;
thetaONGEV          = Result.betathetaONGEVgmm(59:65);
Data.SigmaResults   = thetaONGEV(1:Data.Nsigma);
Data.PhoResults     = thetaONGEV(Data.Nsigma+1:end);

% Y = ln(sj) - ln(s0)

delta0     = Data.Y;

% Before exit

save mvalold delta0
Data.deltONGEV  = delta(thetaONGEV,Data);

Data.deltaEX    = Data.deltONGEV - (alpha ./mean(Data.gdppercapita)) * Data.price;
[sjBEFORE,~]    = ONGEVShareDelta(Data.deltONGEV,Data);

% After exit
Select.SigmaResults   = thetaONGEV(1:Data.Nsigma);
Select.PhoResults     = thetaONGEV(Data.Nsigma+1:end);
Select.alpha          = alpha;

% Delta exogenous wrt price/income
Data.delta      = Data.deltONGEV - (alpha ./mean(Data.gdppercapita)) * Data.price;
derSharePrice   = deriv(@ONGEVSharePrice,Data.price,Data);

% Find closest substitutes
elastONGEV      = bsxfun(@rdivide,Data.price',sjBEFORE) .* derSharePrice;

ElastLuxuryCompact  = elastONGEV(:,find(AllRemove==1));

% In the same class
[test,Index] = sort(ElastLuxuryCompact(:,1));

% % In different classes
% ElastLuxuryCompact  = elastONGEV(:,find(AllRemove==1));
% Sub = ElastLuxuryCompact(find(Data.Fclass==3),:);
% [test,ClosestIndex] = max(Sub);


deltmin         = Data.ownership .* derSharePrice';
mcost           = Data.price + (deltmin\sjBEFORE);
Data.mcost      = mcost;

% Remove the product and recalculate prices
Select.mcost                    = Data.mcost;
Select.mcost(AllRemove,:)       = [];
Select.deltONGEV                = Data.deltONGEV;
Select.deltONGEV(AllRemove,:)   = [];

Select.delta                    = Select.deltONGEV - (Select.alpha./Select.gdppercapita) .* Select.price;

newprice        = fsolve1(@FOC,Select.price,Select);
Data.newprice   = newprice;

% Shares under new price equilibrium

[skAFTER,skouAFTER] = ONGEVSharePrice(newprice,Select);

SharebyNest0    = [(1 - sum(sjBEFORE));zeros(Data.Nsigma,1)];
SharebyNest1    = [skouAFTER(1);zeros(Data.Nsigma,1)];
DiversionONGEV  = [(skouAFTER(1) - (1 - sum(sjBEFORE)))./sum(AllRemove.*sjBEFORE)*100;zeros(Data.Nsigma,1)];

Select.sj = Data.sj;
Select.sj(AllRemove,:) = [];

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1)   = sum(Select.sj(Select.Fclass==j));
    SharebyNest1(j+1)   = sum(skAFTER(Select.Fclass==j));
    DiversionONGEV(j+1) = (- (sum(skAFTER(Select.Fclass==j)) - sum(Select.sj(Select.Fclass==j)))./(-sum(AllRemove.*sjBEFORE)))*100;
end    

% Percent change
MktShareChangeONGEV = ((SharebyNest1./SharebyNest0) - 1)*100;
