%% 0. Upload data
clear;
load WTSeg.mat;

% France 2007 -> Identifier = 11

% Germany 2011

Identifier = 28;

clearvars -except Data Identifier
load ResultEstimation.mat;

%% Select one year/market combination: Germany 2011. Yema=28

Data.misidhelp = Data.misidhelp(:,Identifier);

Data.co2 = Data.co2(Data.yema==Identifier,:);
Data.co2(Data.misidhelp==0) = [];

Data.weightmat      = Data.weightmat(:,:,Identifier);
Data.logobsshare    = Data.logobsshare(:,Identifier);
Data.logobsshare(Data.misidhelp==0) = [];

Data.prods = sum(Data.misidhelp); % I do not need a balanced panel across markets here, so I delete missings
Data.nmkt  = 1;
Data.nobs  = Data.prods*Data.nmkt;

Data.multidumnest   = Data.multidumnest(:,:,Identifier);
misidhelp2          = [1;Data.misidhelp]; % take outside good into account
Data.multidumnest(misidhelp2==0,:,:) = [];

Data.lsjg       = Data.lsjg(Data.yema==Identifier,:);
Data.lsjg(Data.misidhelp==0) = [];

Data.multidumseg    = Data.multidumseg(:,:,Identifier);
Data.multidumseg(Data.misidhelp==0,:,:) = [];

Data.Fclass      = Data.Fclass(Data.yema==Identifier,:);
Data.Fclass(Data.misidhelp==0) = [];

Data.model          = Data.model(Data.yema==Identifier,:);
Data.model(Data.misidhelp==0) = [];

Data.model_group    = Data.model_group(Data.yema==Identifier,:);
Data.model_group(Data.misidhelp==0) = [];

Data.household      = Data.household(Data.yema==Identifier,:);
Data.household(Data.misidhelp==0) = [];

Data.firm           = Data.firm(Data.yema==Identifier,:);
Data.firm(Data.misidhelp==0) = [];

Data.gdppercapita   = Data.gdppercapita(Data.yema==Identifier,:);
Data.gdppercapita(Data.misidhelp==0) = [];

Data.price          = Data.price(Data.yema==Identifier,:);
Data.price(Data.misidhelp==0) = [];

Data.sj             = Data.sj(Data.yema==Identifier,:);
Data.sj(Data.misidhelp==0) = [];

Data.q              = Data.q(Data.yema==Identifier,:);
Data.q(Data.misidhelp==0) = [];

Data.Y              = Result.Y(Data.yema==Identifier,:);
Data.Y(Data.misidhelp==0) = [];

Data.Nsigma = 5;
Data.Npho   = 2;

multiPrice      = bsxfun(@times,Data.price,Data.multidumseg);

% Define variables that will be used for NL and ONGEV

firmdum         = dummyvar(Data.firm);
Data.firmdum    = firmdum;
Data.ownership  = firmdum*firmdum';

FuelCoeffBgmm   = Result.Bgmm(2);
FuelCoeffONGEV  = Result.betathetaONGEVgmm(2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Simulate a subsidy only for cars below 140co2g/km by

% % Identify cars below 150 co2g/km
% Data.CleanCars = Data.co2<150;
% 
% % Percent eligible cars
% PercentCleanCars    = sum(Data.CleanCars)./size(Data.CleanCars,1);
Data.ownership      = Data.firmdum * Data.firmdum';

%% 1. Nested Logit

alpha           = Result.Bgmm(end-Data.Nsigma);
Data.alpha      = alpha;

% Y = ln(sj) - ln(s0)

Sigma           = Result.Bgmm(end-Data.Nsigma+1:end);
Data.dumseg     = dummyvar(Data.Fclass);
thetaNL         = [Sigma;zeros(Data.Npho,1)];
Data.deltaNL    = Data.Y - (bsxfun(@times,Data.lsjg,Data.dumseg)) * Sigma;

Data.SigmaResults   = thetaNL(1:Data.Nsigma);
Data.PhoResults     = thetaNL(Data.Nsigma+1:end);
% Before subsidy
Data.Subsidy = 0;
[~,sjBEFORE]         = ONGEVShareDelta(Data.deltaNL,Data);
CO2BeforeNL           = sum(sjBEFORE(2:end)./sum(sjBEFORE(2:end)).*Data.co2);

Data.delta      = Data.deltaNL - (Data.alpha./Data.gdppercapita) .* Data.price;
derSharePrice   = deriv(@ONGEVSharePrice,Data.price,Data);
elastNL         = bsxfun(@rdivide,Data.price',Data.sj) .* derSharePrice;

deltmin         = Data.ownership .* derSharePrice';
mcost           = Data.price + (deltmin\Data.sj);
Data.mcost      = mcost;

% Recalculate price after subsidy

incentive = zeros(size(mcost));
incentive(Data.co2<140) = 5000;

Data.Subsidy = incentive;

% Data.Subsidy = -(feebate + incentive);

% Theoretical Subsidy based on co2
% incentive               = zeros(size(mcost));
% incentive(Data.co2<140) = 5000;
% Data.Subsidy            = incentive;

% Data.Subsidy            = 2500;
newprice            = fsolve1(@FOC,Data.price,Data);

% Shares under new price equilibrium
[skAFTER,skouAFTER] = ONGEVSharePrice(newprice,Data);
CO2AfterNL          = sum(skAFTER./sum(skAFTER(2:end)).*Data.co2);
DeltaCO2NL          = (CO2AfterNL - CO2BeforeNL)./CO2BeforeNL;

SharebyNest0 = [sjBEFORE(1);zeros(Data.Nsigma,1)];
SharebyNest1 = [skouAFTER(1);zeros(Data.Nsigma,1)];

sjBEFORE = sjBEFORE(2:end);

QuantBEFORE     = sjBEFORE  .* Data.household;
QuantAFTER      = skAFTER   .*Data.household;

InsideShareBEFORE_NL   = zeros(Data.Nsigma,1);
InsideShareAFTER_NL    = zeros(Data.Nsigma,1);

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1) = sum(sjBEFORE(Data.Fclass==j));
    SharebyNest1(j+1) = sum(skAFTER(Data.Fclass==j));
    InsideShareBEFORE_NL(j) = sum(QuantBEFORE(Data.Fclass==j))./sum(QuantBEFORE);
    InsideShareAFTER_NL(j)  = sum(QuantAFTER(Data.Fclass==j))./sum(QuantAFTER);
end

% Percent change
MktShareChangeNL = ((SharebyNest1./SharebyNest0) - 1)*100;

%% 1. ONGEV
% Product elasticity
alpha               = Result.betathetaONGEVgmm(58);
Data.alpha          = alpha;
thetaONGEV          = Result.betathetaONGEVgmm(59:65);
Data.SigmaResults   = thetaONGEV(1:Data.Nsigma);
Data.PhoResults     = thetaONGEV(Data.Nsigma+1:end);

delta0     = Data.Y;

save mvalold delta0
Data.deltONGEV  = delta(thetaONGEV,Data);
% Before subsidy
Data.Subsidy = 0;
[~,sjBEFORE]         = ONGEVShareDelta(Data.deltONGEV,Data);
CO2BeforeONGEV       = sum((sjBEFORE(2:end)./sum(sjBEFORE(2:end))).*Data.co2);

Data.delta          = Data.deltONGEV  - (alpha ./mean(Data.gdppercapita)) * Data.price;
derSharePrice       = deriv(@ONGEVSharePrice,Data.price,Data);
elastONGEV          = bsxfun(@rdivide,Data.price',Data.sj) .* derSharePrice;

deltmin         = Data.ownership .* derSharePrice';
mcost           = Data.price + (deltmin\Data.sj);
Data.mcost      = mcost;

% Recalculate price after subsidy
% Theoretical Subsidy based on co2
% Data.Subsidy    = incentive;

% Data.Subsidy = - (feebate + incentive);
% Data.Subsidy            = 2500;
 Data.Subsidy   = incentive;
newpriceONGEV   = fsolve1(@FOC,Data.price,Data);

% Shares under new price equilibrium
[skAFTER,skouAFTER] = ONGEVSharePrice(newpriceONGEV,Data);
CO2AfterONGEV          = sum((skAFTER./sum(skAFTER)).*Data.co2);
DeltaCO2ONGEV          = (CO2AfterONGEV - CO2BeforeONGEV)./CO2BeforeONGEV;

SharebyNest0 = [sjBEFORE(1);zeros(Data.Nsigma,1)];
SharebyNest1 = [skouAFTER(1);zeros(Data.Nsigma,1)];

sjBEFORE        = sjBEFORE(2:end);

QuantBEFORE     = sjBEFORE  .* Data.household;
QuantAFTER      = skAFTER   .*Data.household;

InsideShareBEFORE_ONGEV   = zeros(Data.Nsigma,1);
InsideShareAFTER_ONGEV    = zeros(Data.Nsigma,1);

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1) = sum(sjBEFORE(Data.Fclass==j));
    SharebyNest1(j+1) = sum(skAFTER(Data.Fclass==j));
    InsideShareBEFORE_ONGEV(j) = sum(QuantBEFORE(Data.Fclass==j))./sum(QuantBEFORE);
    InsideShareAFTER_ONGEV(j)  = sum(QuantAFTER(Data.Fclass==j))./sum(QuantAFTER);

end

% Percent change
MktShareChangeONGEV     = ((SharebyNest1./SharebyNest0) - 1)*100;
InsideShareChangeONGEV  = InsideShareAFTER_ONGEV - InsideShareBEFORE_ONGEV;