%% 0. Upload data
clear;
load WTSeg.mat;

% France 2007 -> Identifier = 11

Identifier = 11;

% eurokmOLD = eurokm(yema==(Identifier-1),:);
eurokmNOW   = eurokm(yema==Identifier,:);
fuelprice   = gp(yema==Identifier,:);

fuelpriceOLD   = gp(yema==Identifier-1,:);

diesel      = fuel(yema==Identifier,:)==2;
gasoline    = fuel(yema==Identifier,:)==1;

dieselOLD   = fuel(yema==Identifier-1,:)==2;
gasolineOLD = fuel(yema==Identifier-1,:)==1;

dieselprice      = mean(fuelprice(diesel));
gasolineprice    = mean(fuelprice(gasoline));

dieselpriceOLD   = mean(fuelpriceOLD(dieselOLD));
gasolinepriceOLD = mean(fuelpriceOLD(gasolineOLD));

% EurokmOLD = liter * fuel price OLD
eurokmOLD    = ((li(yema==Identifier,:) .* diesel) .* dieselpriceOLD/1000) + ((li(yema==Identifier,:) .* gasoline) .* gasolinepriceOLD/1000);

clearvars -except eurokmOLD eurokmNOW Identifier
load ResultEstimation.mat;

%% Select one year/market combination: Germany 2011. Yema=28

Data.misidhelp = Data.misidhelp(:,Identifier);

eurokmOLD(Data.misidhelp==0) = [];
eurokmNOW(Data.misidhelp==0) = [];

Data.co2 = Data.co2(Data.yema==Identifier,:);
Data.co2(Data.misidhelp==0) = [];

Data.weightmat      = Data.weightmat(:,:,Identifier);
Data.logobsshare    = Data.logobsshare(:,Identifier);
Data.logobsshare(Data.misidhelp==0) = [];

Data.prods = sum(Data.misidhelp); % I do not need a balanced panel across markets here, so I delete missings
Data.nmkt  = 1;
Data.nobs  = Data.prods*Data.nmkt;

Data.multidumnest   = Data.multidumnest(:,:,Identifier);
misidhelp2          = [1;Data.misidhelp]; % take outside good into account
Data.multidumnest(misidhelp2==0,:,:) = [];

Data.lsjg       = Data.lsjg(Data.yema==Identifier,:);
Data.lsjg(Data.misidhelp==0) = [];

Data.multidumseg    = Data.multidumseg(:,:,Identifier);
Data.multidumseg(Data.misidhelp==0,:,:) = [];

Data.Fclass      = Data.Fclass(Data.yema==Identifier,:);
Data.Fclass(Data.misidhelp==0) = [];

Data.model          = Data.model(Data.yema==Identifier,:);
Data.model(Data.misidhelp==0) = [];

Data.model_group    = Data.model_group(Data.yema==Identifier,:);
Data.model_group(Data.misidhelp==0) = [];

Data.household      = Data.household(Data.yema==Identifier,:);
Data.household(Data.misidhelp==0) = [];

Data.firm           = Data.firm(Data.yema==Identifier,:);
Data.firm(Data.misidhelp==0) = [];

Data.gdppercapita   = Data.gdppercapita(Data.yema==Identifier,:);
Data.gdppercapita(Data.misidhelp==0) = [];

Data.price          = Data.price(Data.yema==Identifier,:);
Data.price(Data.misidhelp==0) = [];

Data.sj             = Data.sj(Data.yema==Identifier,:);
Data.sj(Data.misidhelp==0) = [];

Data.q              = Data.q(Data.yema==Identifier,:);
Data.q(Data.misidhelp==0) = [];

Data.Y              = Result.Y(Data.yema==Identifier,:);
Data.Y(Data.misidhelp==0) = [];

Data.Nsigma = 5;
Data.Npho   = 2;

multiPrice      = bsxfun(@times,Data.price,Data.multidumseg);

% Define variables that will be used for NL and ONGEV

firmdum         = dummyvar(Data.firm);
Data.firmdum    = firmdum;
Data.ownership  = firmdum*firmdum';

FuelCoeffBgmm   = Result.Bgmm(2);
FuelCoeffONGEV  = Result.betathetaONGEVgmm(2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% 1. Nested Logit

alpha           = Result.Bgmm(end-Data.Nsigma);
Data.alpha      = alpha;


Sigma           = Result.Bgmm(end-Data.Nsigma+1:end);
Data.dumseg     = dummyvar(Data.Fclass);
thetaNL         = [Sigma;zeros(Data.Npho,1)];

Data.SigmaResults = thetaNL(1:Data.Nsigma);
Data.PhoResults   = thetaNL(Data.Nsigma+1:end);

% Delta net of price
Data.deltaNL  = delta(thetaNL,Data);


% Before subsidy
Data.Subsidy = 0;
[~,sjBEFORE]         = ONGEVShareDelta(Data.deltaNL,Data);

Data.deltaNET     = Data.deltaNL  - FuelCoeffBgmm .* eurokmNOW ;
Data.delta        = Data.deltaNET + FuelCoeffBgmm .* eurokmOLD - (Data.alpha ./mean(Data.gdppercapita)) .* Data.price;

% Recalculate price after subsidy
% Simulate the French feebate


feebate = zeros(size(Data.price));
feebate(Data.co2<100) = 1000;
feebate(Data.co2<120) = 700;
feebate(Data.co2<130) = 200;

feebate(Data.co2>161 & Data.co2<165) = -200;
feebate(Data.co2>166 & Data.co2<200) = -750;
feebate(Data.co2>201 & Data.co2<250) = -1600;
feebate(Data.co2>251) = -2600;

incentive = zeros(size(Data.price));
incentive(Data.co2<150) = 1000;

% Shares under new price equilibrium

Data.Subsidy        = -(feebate + incentive);
[skAFTER,skouAFTER] = ONGEVSharePrice(Data.price,Data);

SharebyNest0 = [sjBEFORE(1);zeros(Data.Nsigma,1)];
SharebyNest1 = [skouAFTER(1);zeros(Data.Nsigma,1)];

sjBEFORE = sjBEFORE(2:end);

QuantBEFORE     = sjBEFORE  .* Data.household;
QuantAFTER      = skAFTER   .*Data.household;

InsideShareBEFORE_NL   = zeros(Data.Nsigma,1);
InsideShareAFTER_NL    = zeros(Data.Nsigma,1);

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1) = sum(sjBEFORE(Data.Fclass==j));
    SharebyNest1(j+1) = sum(skAFTER(Data.Fclass==j));
    InsideShareBEFORE_NL(j) = sum(QuantBEFORE(Data.Fclass==j))./sum(QuantBEFORE);
    InsideShareAFTER_NL(j)  = sum(QuantAFTER(Data.Fclass==j))./sum(QuantAFTER);
end

% Percent change
MktShareChangeNL = ((SharebyNest1./SharebyNest0) - 1)*100;

%% 1. ONGEV
% Product elasticity
alpha               = Result.betathetaONGEVgmm(58);
Data.alpha          = alpha;
thetaONGEV          = Result.betathetaONGEVgmm(59:65);
Data.SigmaResults   = thetaONGEV(1:Data.Nsigma);
Data.PhoResults     = thetaONGEV(Data.Nsigma+1:end);

delta0     = Data.Y;

save mvalold delta0
Data.deltONGEV  = delta(thetaONGEV,Data);

% Before subsidy
[~,sjBEFORE]         = ONGEVShareDelta(Data.deltONGEV,Data);

Data.deltaNET     = Data.deltONGEV - (Data.alpha./Data.gdppercapita) .* Data.price - FuelCoeffONGEV .* eurokmNOW;
Data.delta        = Data.deltaNET + FuelCoeffONGEV .* eurokmOLD;

% Recalculate price after subsidy
Data.Subsidy = - (feebate + incentive);

% Shares under new price equilibrium
[skAFTER,skouAFTER] = ONGEVSharePrice(Data.price,Data);

SharebyNest0 = [sjBEFORE(1);zeros(Data.Nsigma,1)];
SharebyNest1 = [skouAFTER(1);zeros(Data.Nsigma,1)];

sjBEFORE        = sjBEFORE(2:end);

QuantBEFORE     = sjBEFORE  .* Data.household;
QuantAFTER      = skAFTER   .*Data.household;

InsideShareBEFORE_ONGEV   = zeros(Data.Nsigma,1);
InsideShareAFTER_ONGEV    = zeros(Data.Nsigma,1);

for j = 1:(Data.Nsigma)
    SharebyNest0(j+1) = sum(sjBEFORE(Data.Fclass==j));
    SharebyNest1(j+1) = sum(skAFTER(Data.Fclass==j));
    InsideShareBEFORE_ONGEV(j) = sum(QuantBEFORE(Data.Fclass==j))./sum(QuantBEFORE);
    InsideShareAFTER_ONGEV(j)  = sum(QuantAFTER(Data.Fclass==j))./sum(QuantAFTER);

end

% Percent change
MktShareChangeONGEV     = ((SharebyNest1./SharebyNest0) - 1)*100;
InsideShareChangeONGEV  = InsideShareAFTER_ONGEV - InsideShareBEFORE_ONGEV;