function [delta1,i] = delta(theta,data)

% Delta- Contraction Mapping for ONGEV
% Works like Logit apart for the weighting of the difference between
% calculated and observed shares
% Syntax:  f = delta(theta,Data)
% The code is "parallelized" because the delta is calculated by market
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    Mean Value Delta
%
% MAT-files required: mvalold
% Subfunctions: ONGEVShareCalculation
%

% Author: Laura Grigolon

load mvalold
k   = 100;
km  = 1e-8;

deltastart              = delta0;

% Unpack
logobsshare              = data.logobsshare;

i = 0;
while k > km
    % Market Share
    [sk,~]  = ONGEVShareDelta(deltastart,data);
    % Modified contraction mapping for ONGEV
    logsk = log(sk);
    logsk(isinf(logsk))= 0;
    delta1  = deltastart+((1-max(theta))*(bsxfun(@minus,logobsshare,logsk)));
    if max(max(isnan(delta1))) == 1 || max(max(isreal(delta1))) == 0 || max(max(isinf(delta1))) == 1
        disp('No Convergence - delta calculation failed: OVERFLOW')
        break
    end
    if i > 3000
        disp('No Convergence - delta convergence failed - too many iterations')
        break
    else
        k           = max(max(abs(bsxfun(@minus,delta1,deltastart))));
        deltastart  = delta1;
        i = i + 1;
    end
end   
%     disp(['# of iterations for delta convergence:  ' num2str(i)])
    
end