clear;

% 1. Simulation settings
% Vector of seeds that creates datasets
R           =  500;     % Number of MC simulations
seedvec     =  1:100000/R:100000;

% 1.2. TRUE PARAMETER VALUES

% Size of the datasets
nmkt        = 10;                         % number of markets = (# of cities)*(# of quarters)
prods       = 100;                        % number of brands per market
nobs        = nmkt*prods;                 % number of observations
meanc       = [3 3];                      % mean of the generated vectors:
betatrue    = [-5 ; -1];                  % true mean tastes on constant and X1
Nnest       = 5;                          % number of nests without outside good
M           = 2;                          % number of neighbors that you have at each side
Sigma       = 0.5;                        % Sigma has the dimension of the number of nests + OUTSIDE GOOD NEST
Nsigma      = size(Sigma,1);
Pho         = 0;                        % Pho has dimension number of nests (+OUTSIDE GOOD NEST) + M
weight      = [1/3;1/3;1/3];              % Weight of each neighboring nest. This has to sum up to 1 and the dimension is M+1 x 1

% Store results

Coefftrue            = [betatrue;Sigma;Pho];                % true parameters of the model
Coefficients         = NaN(numel(Coefftrue),3,R);
ComputationTime      = NaN(3,R); % 3 estimators (Logit, NL, ONGEV)

ElaSELECT            = NaN(2,2,4,R);

FunctionValue        = NaN(1,3,R);
ExitFlag             = NaN(1,3,R);

% 2. Start Simulation
rng default
ksi     = randn(nobs,1).*0.5;
ksi     = ksi-mean(ksi);                     % rescale to have E[ksi]=0
const   = ones(nobs,1);

%% Set correlation: parameter that determines the extent the segment is informative about the continuous characteristic
corrDstarX1  = 0.8;
stdevDstar   = 1;                                % std. dev. of dstar (does not matter)
stdevX1      = 1;                                % std. dev. of X1

Chelp1       = [stdevDstar^2, corrDstarX1*stdevX1*stdevDstar; corrDstarX1*stdevX1*stdevDstar, stdevX1^2];    %varcovar matrix

for MC = 1:R
    rng(seedvec(MC))
    
    % 3. Simulate data
    disp(['Simulation Numb: ',num2str(MC)])
    
    X1        = zeros(prods,nmkt);
    Dstarhelp = zeros(prods,nmkt);

    % triangular distribution
    
    for j = 1:nmkt
        draws    = rand(prods,1);
        X1(:,j)  = (sqrt(2.*draws)-1) .* (draws<=0.5) + (1-sqrt(2.*(1-draws))) .* (draws >0.5);
    end
    X1          = abs((reshape(X1, [nobs 1]))*3);
    nesthelp    = ceil(Nnest *3* tiedrank(X1) / length(X1)); % generate equally size nests
         
    % make lower nests more numerous
    nest = NaN(nobs,1);
    nest(nesthelp<=5)              = 1;
    nest(nesthelp>5&nesthelp<=9)   = 2;
    nest(nesthelp>9&nesthelp<=12)  = 3;
    nest(nesthelp>12&nesthelp<=14) = 4;
    nest(nesthelp>14)              = 5;
        
    dumseg        = dummyvar(nest);
    multidumseg   = reshape(dumseg',[Nnest,prods,nmkt]);
    multidumseg   = permute(multidumseg,[2 1 3]);
  
    % check if there are empty nests and go to the next iteration if there
    % are (cannot calculate market shares with empty nests)
    test = sum(multidumseg);
    test2 = sum(squeeze(sum(test==0)));
    
    if test2>0
        disp('empty nest - go to next iteration')
        continue
    end
    
    %% Create a panel and sort X1 within each market
    
    mktID = (reshape(repmat(1:nmkt, prods,1 ), 1, [] ))';
    
    PanelMat    = [mktID , X1 , nest, ksi];
    PanelSorted = sortrows(PanelMat,[1 3 2]);
    
    nest = PanelSorted(:,3);
    X1   = PanelSorted(:,2);
    ksi  = PanelSorted(:,4);
   
    %% Add outside good as the inferior quality good: get rid of this part if you do not want the outside good
    nestou              = zeros((prods+1),nmkt);
    nestr               = reshape(nest,[prods nmkt]);
    nestou(2:1:end,:)   = nestr;
    nestou              = reshape(nestou,[(prods+1)*nmkt,1]);
    nestou              = nestou+1;     % this is to get positive matrix indices;
        
    % Number of parameters
    nlin    = size(betatrue,1);    % number of linear parameters
    nrc     = 1;                   % number of random coeficients for Random coefficient Logit
    
    Xexo                = [const X1];
    deltaTrue           = Xexo*betatrue+ksi;
    
    %% Weighting matrix
    weight = weight(:,ones(Nnest+1,1));
    % create weighting matrix
    weightmat = zeros(Nnest+M+1,Nnest+1,1);
    for j = 1:(Nnest+1)
        weightmat(j:1:((M+1)+(j-1)),j) = weight(:,j);
    end
    weightmat = weightmat(:, :, ones(nmkt, 1));
    
    % Market share calculation
    if size(Sigma) > 1
        SigmaNest     = Sigma((nestou),:);
    else
        SigmaNest = Sigma;
    end
    dumnest       = dummyvar(nestou);
    multidumnest  = reshape(dumnest',[Nnest+1,(prods+1),nmkt]);
    multidumnest  = permute(multidumnest,[2 1 3]);
    multiX1      = reshape(X1,[prods,1,nmkt]);
    
    dumseg       = dummyvar(nest);
    multidumseg  = reshape(dumseg',[Nnest,prods,nmkt]);
    multidumseg  = permute(multidumseg,[2 1 3]);
        
    Data.nest           = nestou;
    Data.nmkt           = nmkt;
    Data.prods          = prods;
    Data.nobs           = nobs;
    Data.Sigma          = Sigma;
    Data.SigmaNest      = SigmaNest;
    Data.Pho            = Pho;
    Data.Nnest          = Nnest+1; % include outside good
    Data.M              = M;
    Data.dumnest        = dumnest;
    Data.multidumnest   = multidumnest;
    Data.multidumseg    = multidumseg;
    Data.weightmat      = weightmat;
    Data.Nsigma         = Nsigma;
    Data.nlin           = nlin;
    
    % True Market Shares
    % input delta: no outside good, but prods x nmkt
    deltaTrue           = reshape(deltaTrue,[prods nmkt]);
    [sk,skou,sjg]       = TrueShare(deltaTrue,Data);
      
    logobsshare         = log(sk);
    Data.logobsshare    = logobsshare;
    s0                  = skou(1,:);
    sjg                 = permute(sjg,[1 3 2]);
    sjg                 = reshape(sjg,[nobs Nnest]);
    lnsjg               = log(sjg);
    lnsjg(isinf(lnsjg)) = 0;
    lnsjg               = sum(lnsjg,2);
    y                   = reshape((bsxfun(@minus,log(sk),log(s0))),[nobs 1]);             % log-odds ratio
    
    theta0  = [Sigma;Pho+0.001];
    
    % Optimal instruments
    
    % Delta without ksi
    
    derShareDelta     = zeros(prods,prods,nmkt);
    derShareTheta     = zeros(prods,numel(theta0),nmkt);
    PseudoDelta       = reshape(Xexo*betatrue,[prods 1 nmkt]);
    
    for i=1:nmkt
        PseudoDeltaMA         = PseudoDelta(:,:,i);
        multidumnestMA        = multidumnest(:,:,i);
        weightmatMA           = weightmat(:,:,i);
        Data.multidumnestMA   = multidumnestMA;
        Data.weightmatMA      = weightmatMA;
        Data.PseudoDeltaMA    = PseudoDeltaMA;
        fHandle               = @TrueShareMA;
        derShareDelta(:,:,i)  = numgrad(fHandle,PseudoDeltaMA,Data);
        fHandle               = @PseudoDeltaTrueShare;
        derShareTheta(:,:,i)  = numgrad(fHandle,theta0,Data);
    end
    
    derDeltaTheta   = multiprod((-multinv(derShareDelta)),derShareTheta);
    derDeltaTheta   = permute(derDeltaTheta,[2 1 3]);
    chamberlin      = reshape(derDeltaTheta, [numel(theta0) nobs]);
    chamberlin      = chamberlin';
    
    Z               = [Xexo chamberlin];
    
    Data.alpha      = betatrue(nlin);
    Data.theta      = [Data.Sigma;Data.Pho];
    deltaEX         = Xexo*betatrue + ksi - Data.alpha*X1;
    deltaEX         = reshape(deltaEX,prods,nmkt);
    
    %% True Elasticity
    
    % True product elasticity
   
    [ElaTRUE, DiversionTRUE]     = ElastONGEV(Data.alpha,Data.theta,deltaTrue,multiX1,Data);
    
    ElaSELECT(:,:,4,MC) = mean(ElaTRUE(31:32,31:32,:),3);
    
    %% Misallocation: Take the product(s) with the highest value in nest 1 and wrongly attribute it to nest 2
    MISindex6 =31:prods:nobs;
    MISnest = nest;
    MISnest(MISindex6) = 2;
    
    % Add the outside good as the inferior quality good: get rid of this part if you do not want the outside good
    nestou              = zeros((prods+1),nmkt);
    nestr               = reshape(nest,[prods nmkt]);
    nestou(2:1:end,:)   = nestr;
    nestou              = reshape(nestou,[(prods+1)*nmkt,1]);
    nestou              = nestou+1;     % this is to get positive matrix indices;
    
    dumnest       = dummyvar(nestou);
    multidumnest  = reshape(dumnest',[Nnest+1,(prods+1),nmkt]);
    multidumnest  = permute(multidumnest,[2 1 3]);

    dumseg       = dummyvar(nest);
    multidumseg  = reshape(dumseg',[Nnest,prods,nmkt]);
    multidumseg  = permute(multidumseg,[2 1 3]);
    
    Data.nest           = nestou;  
    Data.dumnest        = dumnest;
    Data.multidumnest   = multidumnest;
    Data.multidumseg    = multidumseg;   
    
    % 6 Estimation
    
    %% Logit
    % Weighting Matrix
    W           = (Z'*Z)\eye(size(Z,2));
    BLog        = ((Xexo'*Z*W*Z'*Xexo)\eye(size(Xexo,2)))*(Xexo'*Z*W*Z'*y);

    est         = y-Xexo*BLog;

    
    deltL       = y;
    alpha       = BLog(nlin);
    theta       = 0;
    
    % Logit elasticity wrt X1
    [ElaLogit, DiversionLogit]  = ElastLogit(alpha,theta,deltL,multiX1,Data); 

    % Function value
    FctValLog   = (est' * Z) * W * (Z' * est);
    
    % Store the results
    Coefficients(:,1,MC)          = [BLog;nan;nan];
    ComputationTime(1,MC)         = nan;
    ElaSELECT(:,:,1,MC)           = mean(ElaLogit(31:32,31:32,:),3);
    FunctionValue(:,1,MC)         = FctValLog;
    ExitFlag(:,1,MC)              = nan;
    
    %% Nested Logit
    XNestedLog  = [Xexo lnsjg];
    BNLog       = ((XNestedLog'*Z*W*Z'*XNestedLog)\eye(size(XNestedLog,2)))*(XNestedLog'*Z*W*Z'*y);
    est         = y-XNestedLog*BNLog;

    deltNL     = y - BNLog(nlin+1)*lnsjg;
    % Nested Logit elasticity wrt X1
    alpha      = BNLog(nlin);
    Data.alpha = alpha;
    theta      = [BNLog(nlin+1);0];
    [ElaNL, DiversionNL]     = ElastNestedLogit(alpha,theta,deltNL,multiX1,Data);

    
    % Function value
    FctValNL   = (est' * Z) * W * (Z' * est);
    AICNLog    = nobs * FctValNL - 2 * (size(Z,2)-size(BNLog,1));
    BICNLog    = nobs * FctValNL - (size(Z,2)-size(BNLog,1)) * log(nobs);
    
    % Store the results
    Coefficients(:,2,MC)          = [BNLog;nan];
    ComputationTime(2,MC)         = nan; 
    ElaSELECT(:,:,2,MC)           = mean(ElaNL(31:32,31:32,:),3);  
    FunctionValue(:,2,MC)         = FctValNL;
    ExitFlag(:,2,MC)              = nan;
    
    %% ONGEV
    
    % Some data that speeds up computations
    xzwz            = Xexo'*Z*W*Z';
    Data.xzwz       = xzwz;
    xzwzx           = xzwz * Xexo;
    invxzwzx        = xzwzx\eye(size(xzwzx,2));
    Data.invxzwzx   = inv(xzwzx);
    Data.Z          = Z;
    Data.W          = W;
    Data.Xexo       = Xexo;
    
    t1      = cputime;
    
    % OPTIMIZATION
    % Options:see Knitro Option File KnitroBLP.opt
    % Pass Data into gmm using anonymous functions
    theta0  = [0.6;0.1];
    delta0  = deltaTrue;
    save mvalold delta0
     
    angmmONGEV  = @(theta0)gmmONGEV(theta0,Data);
    
    options = optimset('Algorithm','sqp',...
        'GradObj','off','TolCon',1E-8,...
        'TolFun',1E-8,'TolX',1E-8,...
        'Hessian', 'off','DerivativeCheck','off','FinDiffType','central');
    Aeq = []; 
    beq = [];

    A =[];
    b =[];
    
    x_L     = zeros(size(theta0));       % lower bound (for nesting parameters to avoid numerical problems when sigma/pho gets negative)
    x_U     = ones(size(theta0))*0.975;   % upper bound (for nesting parameters to avoid numerical problems when sigma/pho gets close to 1)
   
    [thetaONGEV, FctValONGEV, exitflagONGEV,outputONGEV] = fmincon(angmmONGEV,theta0,[],[],[],[], x_L, x_U, [], options);
  
    
    deltONGEV       = delta(thetaONGEV,Data);
    Data.deltONGEV  = deltONGEV;
    deltONGEV       = reshape(deltONGEV,[nobs 1]);
    betaONGEV       = invxzwzx*(xzwz*deltONGEV);
    th12ONGEV       = [betaONGEV ; thetaONGEV];

    cputimegmm      = cputime-t1;
    
    % ONGEV elasticity wrt X1
    alpha      = th12ONGEV(nlin);
    Data.alpha = alpha;
    [ElaONGEV, DiversionONGEV]     = ElastONGEV(alpha,thetaONGEV,Data.deltONGEV,multiX1,Data);
    
    % Store the results
    Coefficients(:,3,MC)          = th12ONGEV;
    ComputationTime(3,MC)         = cputimegmm;
    ElaSELECT(:,:,3,MC)           = mean(ElaONGEV(31:32,31:32,:),3);   
    FunctionValue(:,3,MC)         = FctValONGEV;
    ExitFlag(:,3,MC)              = exitflagONGEV;
    
    savefile = 'ONGEVvsNL_misclas.mat';
    save    (savefile, 'Coefftrue', 'Coefficients', 'ComputationTime',...
        'ElaSELECT',...
        'FunctionValue', 'ExitFlag')
end

% Drop draws for which ONGEV did not converge correctly

Coefficients(Coefficients==0)=NaN;

ElaSELECT(:,:,:,~isfinite(Coefficients(4,3,:)))     = [];
Coefficients(:,:,~isfinite(Coefficients(4,3,:)))    = [];

ElaSELECT(:,:,:,(Coefficients(3,3,:)<Coefficients(4,3,:))) =[];
Coefficients(:,:,Coefficients(3,3,:)<Coefficients(4,3,:)) =[]; % Drop pho>sigma

% Analysis results
MeanElaSELECT           = nanmean(ElaSELECT,4)
MeanCoefficient         = nanmean(Coefficients,3)
EmpiricalStErrors       = nanstd(Coefficients,0,3);
ElaSELECTAvg            = nanmean(ElaSELECT,4)
BiasNL_Elast            = nanmean(ElaSELECT(:,:,2)-ElaSELECT(:,:,4),4)
BiasONL_Elast           = nanmean(ElaSELECT(:,:,3)-ElaSELECT(:,:,4),4)

xlswrite('ONGEVvsNL_misclass',ElaSELECTAvg(:,:,4), 'TrueElast','A1')
xlswrite('ONGEVvsNL_misclass',BiasNL_Elast, 'BiasNL_Elast','A1')
xlswrite('ONGEVvsNL_misclass',BiasONL_Elast, 'BiasONL_Elast','A1')
