function [sk,skou]=ONGEVShareCalculation(theta,delta,Data)

% ONGEVShareCalculation - Ordered Logit share calculation
% Syntax:   [sk]=ONGEVShareCalculation(thetaNL,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sk    = market shares
% Subfunctions: none
%

% Author: Laura Grigolon
% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
multidumnest    = Data.multidumnest;
M               = Data.M;
Nnest           = Data.Nnest;
weightmat       = Data.weightmat;
Nsigma          = Data.Nsigma;

% TrueShare - Nested Logit share calculation using true parameter values
% Refer to excel for a layout of the procedure

sigmaNest   = theta(1:Nsigma);
sigmaNest   = repmat(sigmaNest,[(Nnest-1) 1]);
sigmaNest   = [0;sigmaNest];
pho         = theta(Nsigma+1:end);

%% Add outside good in delta
deltaou             = zeros((prods+1),nmkt);
delta               = reshape(delta,[prods nmkt]);
deltaou(2:1:end,:)  = delta;
deltaou             = reshape(deltaou,[(prods+1)*nmkt,1]);

%%
numer1      = exp(deltaou);
% use multidimensional arrays
numer1      = reshape(numer1,[prods+1,1,nmkt]);
numer1      = bsxfun(@times,numer1,multidumnest);

numer1    = bsxfun(@power,numer1,(1./(1-sigmaNest')));

denom1      = nansum(numer1,1);
qkn         = bsxfun(@rdivide,numer1,denom1);

% create Z matrix
Z = bsxfun(@power,denom1,(1-sigmaNest')); % Z = 1 x #Nests x #Markets
Z = repmat(Z,[M+1 1 1]);

Zmat = zeros(Nnest+M, Nnest, nmkt);

for j=(1:Nnest)
    Zmat(j:1:((M+1)+(j-1)),j,:) = Z(:,j,:);
end

numer2 = weightmat.*(bsxfun(@power,Zmat,1./(1-pho)));
denom2 = sum(numer2,2);
qnBr   =  bsxfun(@rdivide, numer2, denom2);

numer3    = bsxfun(@power,denom2,(1-pho));
denom3    = sum(numer3);
qBr       = bsxfun(@rdivide, numer3, denom3);

partial   = bsxfun(@times,qnBr,qBr);
partial   = reshape(partial,[M+1,1,(M+Nnest)*Nnest/(M+1) nmkt]);
partial2  = partial(partial>0);
partial   = reshape(partial2,size(partial,1),size(partial,2),Nnest,size(partial,4));
partial   = permute(partial,[2 3 1 4]);

qkn   = repmat(qkn,[1 M+1 1]);
qkn   = reshape(qkn,[prods+1 Nnest M+1 nmkt]);

sk   = bsxfun(@times,qkn,partial);
sk   = sum(sk,2);
sk   = sum(sk,3);
%2D  = prods X mkt
skou = squeeze(sk);
% remove outside good
sk   = skou(2:end,:);

end