function [sk,skou,sjg]=TrueShare(delta,Data)

% TrueShare - Nested Logit share calculation using true parameter values
% Syntax:   [sij sijg sj sjg s0 numer1 denom1 numer2 denom2]=TrueShare(delta,Data)
%
% Inputs:
%    input1 - Mean value delta
%    input2 - Data
%
% Outputs:
%   qkn
%   qnBr
%   qBr
%   sk
% Subfunctions: none

% Author: Laura Grigolon

% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
sigma           = Data.Sigma;
pho             = Data.Pho;
sigmaNest       = Data.SigmaNest;
multidumnest    = Data.multidumnest;
M               = Data.M;
Nnest           = Data.Nnest;
weightmat       = Data.weightmat;

% TrueShare - Nested Logit share calculation using true parameter values
% Refer to excel for a layout of the procedure

%% Add outside good in delta
deltaou             = zeros((prods+1),nmkt);
deltaou(2:1:end,:)  = delta;
deltaou             = reshape(deltaou,[(prods+1)*nmkt,1]);
%%
numer1    = (exp(deltaou)).^(1./(1-sigmaNest));
% use multidimensional arrays
numer1    = reshape(numer1,[prods+1,1,nmkt]);
numer1    = bsxfun(@times,numer1,multidumnest);
denom1    = sum(numer1,1);
qkn       = bsxfun(@rdivide,numer1,denom1);
% eliminate the outside good
sjg       = qkn(2:end,2:end,:);

% create Z matrix
Z = bsxfun(@power,denom1,(1-sigma')); % Z = 1 x #Nests x #Markets
Z = repmat(Z,[M+1 1 1]);

Zmat = zeros(Nnest+M, Nnest, nmkt);

for j=(1:Nnest)
    Zmat(j:1:((M+1)+(j-1)),j,:) = Z(:,j,:);
end

numer2    = weightmat.*(bsxfun(@power,Zmat,1./(1-pho)));
denom2    = sum(numer2,2);
qnBr      =  bsxfun(@rdivide, numer2, denom2);

numer3    = bsxfun(@power,denom2,(1-pho));
denom3    = sum(numer3);
qBr       = bsxfun(@rdivide, numer3, denom3);

partial   = bsxfun(@times,qnBr,qBr);
partial   = reshape(partial,[M+1,1,(M+Nnest)*Nnest/(M+1) nmkt]);
partial2  = partial(partial>0);
partial   = reshape(partial2,size(partial,1),size(partial,2),Nnest,size(partial,4));
partial   = permute(partial,[2 3 1 4]);

qkn   = repmat(qkn,[1 M+1 1]);
qkn   = reshape(qkn,[prods+1 Nnest M+1 nmkt]);

sk   = bsxfun(@times,qkn,partial);
sk   = sum(sk,2);
sk   = sum(sk,3);
%2D  = prods X mkt
skou = squeeze(sk);
% remove outside good
sk   = skou(2:end,:);
end