function [ElaONGEV, Diversion]=ElastONGEV(alpha,theta,delta,characteristic,Data)

% ElastNestedLogit - Calculate price elasticities for Nested Logit models
% Syntax:  f=ElastLogit(alpha,theta,delta,Data)
%
% Inputs:
%    input1 - alpha     = price coefficient
%    input2 - thetaNL   = random coefficient
%    input1 - delta     = mean value
%    input2 - Data
%
% Outputs:
%    prods x prods elasticity matrix
%
% Subfunctions: NLShareCalculation 
%
% Author: Laura Grigolon
% August 2015;

% Unpack
prods            = Data.prods;
nmkt             = Data.nmkt;

Data.alphaResults   = alpha;
Data.SigmaResults   = theta(1);
Data.PhoResults     = theta(2);

[sj,~]  = ONGEVShareCalculation(theta,delta,Data);
delta             = reshape(delta,[prods 1 nmkt]);
derShareDelta     = zeros(prods,prods,nmkt);
for i=1:nmkt
    % Market Shares
    multidumnestMA        = Data.multidumnest(:,:,i);
    weightmatMA           = Data.weightmat(:,:,i);
    Data.multidumnestMA   = multidumnestMA;
    Data.weightmatMA      = weightmatMA;
    deltaMA               = delta(:,:,i);
    fHandle               = @ONGEVShareMA;
    derShareDelta(:,:,i)  = numgrad(fHandle,deltaMA,Data);
end

sj              = reshape(sj,[prods 1 nmkt]);
PriceOverShare  = multiprod((1/sj),permute(characteristic,[2 1 3]));

ElaONGEV = bsxfun(@times,(derShareDelta*alpha),PriceOverShare);

OwnDeriv        = permute((reshape(diagnd(derShareDelta),[prods,1,nmkt])),[2 1 3]);
Diversion       = bsxfun(@times,(derShareDelta),(1/OwnDeriv));

end
