function [sj, sij]=LogitShareCalculation(theta,delta,Data)

% LogitShareCalculation
% Syntax:   [sj sij]=LogitShareCalculation(theta,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sij    = individual market shares
%    sj     = market shares
%
% Subfunctions: none
%

% Author: Laura Grigolon
% August 2012;

%% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
ndraws          = Data.ndraws;

% Market share calculation
delta           = reshape(delta,[Data.nobs,1]);

mu1 = Data.xvu1  .* theta(1);
mu2 = Data.xvu2  .* theta(2);
mu3 = Data.xvu3  .* theta(3);

mu  = mu1 + mu2 + mu3;

mudel   = delta + mu;
numer1  = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt,ndraws]);
denom1          = 1 + sum(numer1);
sij             = bsxfun(@rdivide,numer1,denom1);
sj              = mean(sij,4);
end