function [sj, sij]=LogitSharePrice(selPrice,Data)

% LogitShareCalculation
% Syntax:   [sj sij]=LogitShareCalculation(theta,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sij    = individual market shares
%    sj     = market shares
%
% Subfunctions: none
%

% Author: Laura Grigolon
% August 2012;

%% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
xvu             = Data.xvu;
qweightrprods   = Data.qweightrprods;
nrc             = Data.nrc;

% Market share calculation
delta   = Data.deltaEX + Data.alpha.* selPrice;
delta   = reshape(delta,prods*nmkt,1);

mu = zeros(prods*nmkt,Nnodes);

for i = 1:nrc
    mu = mu + xvu(:,(i-1)*Nnodes+1:Nnodes*i).*Data.theta(i);
end    
mudel   = bsxfun(@plus,delta,mu);
numer1  = exp(mudel);

numer1  = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt,Nnodes]);
denom1          = 1 + sum(numer1);
sij             = bsxfun(@rdivide,numer1,denom1);
sj              = sum((bsxfun(@times,sij,qweightrprods)),4);
end