function [sj, sij]=LogitSharePriceMA(selPrice,Data)

% LogitShareCalculation
% Syntax:   [sj sij]=LogitShareCalculation(theta,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sij    = individual market shares
%    sj     = market shares
%
% Subfunctions: none
%

% Author: Laura Grigolon
% August 2012;

%% Unpack
prods           = Data.prods;
nmkt            = 1;
ndraws          = Data.ndraws;

% Market share calculation
delta   = Data.deltaEX(1:prods,:) + Data.alpha.* selPrice(1:prods,:);
delta   = reshape(delta,prods*nmkt,1);

mu1 = Data.xvu1(1:prods)  .* Data.RandCoeff(1);
mu2 = Data.xvu2(1:prods)  .* Data.RandCoeff(2);
mu3 = Data.xvu3(1:prods)  .* Data.RandCoeff(3);

mu      = mu1+ mu2+ mu3;
mudel   = delta + mu;
numer1  = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt,ndraws]);
denom1          = 1 + sum(numer1);
sij             = bsxfun(@rdivide,numer1,denom1);
sj              = mean(sij,4);
end