function [sj, sjg, s0]=NLShareCalculation(thetaNL,delta,Data)

% NLShareCalculation - Nested Logit share calculation
% Syntax:   [sij sijg sj sjg s0 numer1 denom1 numer2 denom2]=NLShareCalculation(thetaNL,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    sij    = individual market shares
%    sijg   = individual conditional share of choosing good j from group g
%    sj     = market shares
%    sjg    = conditional share of choosing good j from group g 
%    s0     = outside good
%    numer1 = numerator1
%    denom1 = denominator1
%    numer2 = numerator 2
%    denom2 = denominator 2
%
% Subfunctions: none
%

% Author: Laura Grigolon
% August 2012;

% Unpack
prods           = Data.prods;
nmkt            = Data.nmkt;
ndraws          = Data.ndraws;
multidumseg     = Data.multidumseg;
Nsigma          = Data.Nsigma;

Sigseg  = thetaNL(1:Nsigma);
mudel   = delta./(1-Sigseg);
numer1  = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt]);
numer1          = bsxfun(@times,numer1,multidumseg);
denom1          = sum(numer1,1);

numer2          = denom1.^(1-Sigseg);
denom2          = 1 + sum(numer2,2);                         % sum across segments within a market (second dimension)


sjg              = bsxfun(@rdivide,numer1,denom1);

sg               = bsxfun(@rdivide,numer2,denom2);

sj               = bsxfun(@times,sjg,sg);

s0                = 1 ./ denom2;

end