function [sj]=NLSharePriceMA(selPrice,Data)

% Nested Logit ShareCalculation
% Syntax:   [sk,skou]=NLSharePriceMA(selPrice,Data)
%
% Inputs:
%    input1 - Price
%    input2 - Data
%
% Outputs:
%    sk    = market shares
% Subfunctions: none
%

% Author: Laura Grigolon
% Unpack

% Unpack
prods           = Data.prods;
nmkt            = 1;
multidumseg     = Data.multidumseg(:,:,1);
Sigseg          = Data.Sigseg;

% Market share calculation
delta  = Data.deltaEXMA + Data.alpha.* selPrice;

mudel  = delta ./(1-Sigseg);
numer1 = exp(mudel);

% use multidimensional arrays
numer1          = reshape(numer1,[prods,1,nmkt]);
numer1          = bsxfun(@times,numer1,multidumseg);
denom1          = sum(numer1,1);

numer2          = denom1.^(1-Sigseg);
denom2          = 1 + sum(numer2,2);                         % sum across segments within a market (second dimension)

sijg              = bsxfun(@rdivide,numer1,denom1);
sijg(isnan(sijg)) = 0;

sig               = bsxfun(@rdivide,numer2,denom2);
sig(isnan(sig))   = 0;

sij               = bsxfun(@times,sijg,sig);
sij(isnan(sij))   = 0;

sj                = mean(sij,4);
sj(isnan(sj))     = 0;

sj                = squeeze(sum(sj,2));

end