function [delta1,i] = deltaLogit(theta,Data)

% DeltaNL - Contraction Mapping for Logit
% Syntax:  f = deltaLogit(theta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    Mean Value Delta
%
% MAT-files required: mvalold
% Subfunctions: LogitShareCalculation
%

% Author: Laura Grigolon
% August 2012;

load mvalold
k   = 100;
km  = 1e-14;
i   = 0;
if max(max(isnan(delta0)))  == 1
    deltastart         =  zeros(Data.prods,Data.nmkt);
else
    deltastart         = delta0;
end

% Run all markets at the same time in parallel

while k > km
    % Market Share    
    [sh, ~] = LogitShareCalculation(theta,deltastart,Data);
    sh = squeeze(sh);
    % Contraction Mapping
    delta1  = deltastart + bsxfun(@minus,Data.logobsshare,log(sh));
    if max(max(isnan(delta1))) == 1 || max(isreal(delta1)) == 0
%         disp('No Convergence - delta calculation failed: OVERFLOW')
        break
    end
    if i > 2500
%          disp('No Convergence - delta convergence failed - too many iterations')
        break
    else
    k           = max(max(abs(bsxfun(@minus,delta1,deltastart))));
    deltastart  = delta1;
    i = i + 1;
    end
end

% disp(['# of iterations for delta convergence:  ' num2str(i)])

if isreal(delta1)   == 0
    delta0          =  zeros(Data.prods,Data.nmkt);
else
    delta0 = delta1;
end