 function f     = jacobLogit(theta,delta,Data)

 % jacobLogit - This function is calculate the derivative of the mean value
% delta wrt the parameter using the implicit function theorem (see Nevo
% http://faculty.wcas.northwestern.edu/~ane686/supplements/Ras_guide_appendix.pdf
% 
% 
% jacobLogit(theta,delta,Data)
%
% Inputs:
%    input1 - Non linear parameters
%    input2 - Data
%
% Outputs:
%    Derivative of Delta wrt theta 
%
% Subfunctions: LogitShareCalculation; multinv; multiprod
%

% Author: Laura Grigolon
% August 2012;
 
% Unpack
prods            = Data.prods;
multixvu1        = Data.multixvu1;
multixvu2        = Data.multixvu2;
multixvu3        = Data.multixvu3;

% Market Shares
[~, sij]        = LogitShareCalculation(theta,delta,Data);

% Jacobian
% remember that sij is prod x 1 x mkt x nodes

% derivative of shares wrt to delta
part1           = bsxfun(@times,sij,eye(prods));          % diagonal in multiple dimensions
sijtransp       = permute(sij,[2 1 3 4]);
part2           = multiprod(sij,sijtransp);
derShareDeltij  = part1 - part2;
derShareDelta   = mean(derShareDeltij,4);


% derivative of shares with respect to first random coefficient
part1RC1           = bsxfun(@times,multixvu1,sij);
part2RC1           = bsxfun(@times,sum(part1RC1),sij);
derShareRC1        = part1RC1-part2RC1;
derShareTheta1     = mean(derShareRC1,4);

% derivative of shares with respect to second random coefficient

part1RC2           = bsxfun(@times,multixvu2,sij);
part2RC2           = bsxfun(@times,sum(part1RC2),sij);
derShareRC2        = part1RC2-part2RC2;
derShareTheta2     = mean(derShareRC2,4);

% derivative of shares with respect to third random coefficient

part1RC2           = bsxfun(@times,multixvu3,sij);
part2RC2           = bsxfun(@times,sum(part1RC2),sij);
derShareRC3        = part1RC2-part2RC2;
derShareTheta3     = mean(derShareRC3,4);

derShareTheta      = [derShareTheta1 derShareTheta2 derShareTheta3];

derDeltaTheta      = multiprod((-multinv(derShareDelta)),derShareTheta);
derDeltaTheta      = [(reshape(derDeltaTheta(:,1,:),[Data.nobs 1])),(reshape(derDeltaTheta(:,2,:),[Data.nobs 1])),(reshape(derDeltaTheta(:,3,:),[Data.nobs 1]))]; % 2D market shares

f = derDeltaTheta;

 end